/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#import <UIKit/UIKit.h>
#import "AccountView.h"
#import "UIController.h"
#import "FunambolAPI.h"
#import "base/util/StringBuffer.h"
#import "customs.h"
#import "FastSyncMethodsView.h"
#import "AboutView.h"
#import "URLQueryStringParser.h"

@interface AccountView (Private)
- (UITextField*)buildTextFieldWithFrame:(CGRect)aFrame text:(NSString*)text keyboardType:(UIKeyboardType)keyboardType returnKeyType:(UIReturnKeyType)returnKeyType;
- (NSString*)friendlySyncValue;
- (void)updateSyncValue;
- (void)saveUsername:(NSString*)u;
- (void)savePassword:(NSString*)p forUsername:(NSString*)u;
@end

@implementation AccountView

@synthesize credentials, syncValue;

- (id)init {
	if (self = [super initWithStyle:UITableViewStyleGrouped]) {
		
		self.title = NSLocalizedString(@"Settings",@"Settings");
		showerror = true;		
    }

    return self;
}

- (void)noCredsAlert:(NSString*)text {
    UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Alert", @"Alert")
													 message:text
													delegate:self
										   cancelButtonTitle:NSLocalizedString(@"OK", @"OK")
										   otherButtonTitles:nil] autorelease];
    [alert show];
}

- (void)retrieveCredentials
{
	NSURL *credentialsURL = [NSURL URLWithString:@"https://one.ubuntu.com/phones/creds/ios?scheme=x-ubuntuone-contacts"];
	[[UIApplication sharedApplication] openURL:credentialsURL];
}

- (void)parseCredentials
{
	if (credentials)
	{
		URLQueryStringParser *queryStringParser = [[URLQueryStringParser alloc] initWithURL:[NSURL URLWithString:credentials]];
		NSString *user = [queryStringParser queryStringValueForKey:@"u"];
		NSString *pass = [queryStringParser queryStringValueForKey:@"p"];
		if (user != nil)
		{
			[self saveUsername:user];
		}

		if (pass != nil)
		{
			[self savePassword:pass forUsername:user];
		}
		[queryStringParser release];
	}
}

- (BOOL) hasValidCredentials
{
    [[FunambolAPI getInstance] loadSyncConfig:&URL username:&username password:&password];
	
	if ([username isEqualToString:@""] || username == nil ||
		[password isEqualToString:@""] || password == nil) {
		return NO;
	}
	
	return YES;
}

- (void) viewDidLoad{
    [super viewDidLoad];

	[self parseCredentials];
	
	signInOutButton = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"SignInOutButton"];
	signInOutButton.textLabel.text = [self hasValidCredentials] ? NSLocalizedString(@"Sign Out of Ubuntu One",@"Sign Out button in AccountView.mm") : NSLocalizedString(@"Sign In to Ubuntu One",@"Sign In button in AccountView.mm");
    signInOutButton.textLabel.textAlignment = UITextAlignmentCenter;

	aboutButton = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"AboutButton"];
	aboutButton.textLabel.text = NSLocalizedString(@"About Ubuntu One",@"About button in AccountView.mm");
    aboutButton.textLabel.textAlignment = UITextAlignmentCenter;

    [self.tableView reloadData];
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Close",@"") style:UIBarButtonItemStyleDone target:self action:@selector(close:)] autorelease];
	
	[self updateSyncValue];
	[self.tableView reloadData];
}

- (IBAction)close:(id)sender
{
	[self dismissModalViewControllerAnimated:YES];
}

- (void) viewWillDisappear:(BOOL) animated{
    showerror = true;
	[self.view endEditing:YES];
}

- (void) reload {
}

- (UITableViewCell *)tableView:(UITableView *)accountTable cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	switch (indexPath.section)
	{
		case 0: //Sign In/Out Button
		{
			signInOutButton.textLabel.text = [self hasValidCredentials] ? NSLocalizedString(@"Sign Out of Ubuntu One",@"Sign Out button in AccountView.mm") : NSLocalizedString(@"Sign In to Ubuntu One",@"Sign In button in AccountView.mm");

			return signInOutButton;
		}
		case 1: //Sync Direction
		{
			static NSString* identifier = @"SyncWayIdentifier";
			
			UITableViewCell *cell = [self.tableView dequeueReusableCellWithIdentifier:identifier];
			
			if (nil == cell)
			{
				cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:identifier] autorelease];
				cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
			}
			
			cell.textLabel.text = NSLocalizedString(@"Sync Direction",@"Sync Direction");
			cell.detailTextLabel.text = [self friendlySyncValue];
			
			return cell;
			
			break;
		}
		case 2: //About
		{
			return aboutButton;
		}
		default:
		{
			return nil;
			break;
		}
	}
}

- (NSInteger)tableView:(UITableView *)accountTable numberOfRowsInSection:(NSInteger)section {
	return 1;
}

- (CGFloat)tableView:(UITableView *)accountTable heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
	return 45;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)accountTable
{
	return 3;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
	switch (section)
	{
		case 0:
		{
			return [self hasValidCredentials] ? NSLocalizedString(@"Sign Out",@"") : NSLocalizedString(@"Sign In",@"");
			break;
		}
		case 1:
		{
			return NSLocalizedString(@"Sync Settings",@"");
			break;
		}
		default:
		{
			return nil;
			break;
		}
	}
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	if (0 == indexPath.section)
	{
		if ([self hasValidCredentials])
		{
			// Log out by nilling out our credentials, username, and password?
			[tableView deselectRowAtIndexPath:indexPath animated:YES];
			credentials = nil;
			username = nil;
			[self saveUsername:@""]; // Hack?
			password = nil;
			[self savePassword:@"" forUsername:@""]; // Double Hack?

			[tableView reloadData];
		}
		else
		{
			[tableView deselectRowAtIndexPath:indexPath animated:YES];
			[self retrieveCredentials];
		}
	} else if (1 == indexPath.section) {
		FastSyncMethodsView* syncMethodsViewController = [[[FastSyncMethodsView alloc] init] autorelease];
		[syncMethodsViewController setSyncSourceName:@"addressbook"];
		[syncMethodsViewController setSyncMode:self.syncValue];
		[[self navigationController] pushViewController:syncMethodsViewController animated:YES];
	} else if (2 == indexPath.section) {
		AboutView *about = [[[AboutView alloc] init] autorelease];
		[self.navigationController pushViewController:about animated:YES];
	}
}

#pragma mark -
#pragma mark UITextFieldDelegate

- (BOOL)textFieldShouldReturn:(UITextField *)textField{
	return YES;
}

- (void)dealloc {
    // TODO dealloc all items

    [URL release];

	self.credentials = NULL;
	
	self.syncValue = NULL;
	
	[signInOutButton release];
	[aboutButton release];

    [username release];
    [password release];
    [super dealloc];
}

@end

@implementation AccountView (Private)

- (void)saveUsername:(NSString*)u
{
	[[FunambolAPI getInstance] saveUsernameConfig:u];
	[SFHFKeychainUtils storeUsername:UBUNTU_ONE_DUMMY_USER_NAME andPassword:u forServiceName:UBUNTU_ONE_SERVICE_NAME updateExisting:YES error:nil];
}

- (void)savePassword:(NSString*)p forUsername:(NSString*)u
{
	[[FunambolAPI getInstance] savePasswordConfig:p];
	[SFHFKeychainUtils storeUsername:u andPassword:p forServiceName:UBUNTU_ONE_SERVICE_NAME updateExisting:YES error:nil];
}

- (UITextField*)buildTextFieldWithFrame:(CGRect)aFrame text:(NSString*)text keyboardType:(UIKeyboardType)keyboardType returnKeyType:(UIReturnKeyType)returnKeyType
{
	UITextField *field = [[UITextField alloc] initWithFrame:aFrame];
	field.borderStyle = UITextBorderStyleNone;
	field.font = [UIFont systemFontOfSize:15];
	field.exclusiveTouch = YES;
	field.autocorrectionType = UITextAutocorrectionTypeNo;
	field.text = text;
	field.keyboardType = keyboardType;
	field.returnKeyType = returnKeyType;
	field.clearButtonMode = UITextFieldViewModeNever;
	field.autocapitalizationType = UITextAutocapitalizationTypeNone;
	field.textColor = [UIColor colorWithRed:FUNBLUE_RGB_RED green:FUNBLUE_RGB_GREEN blue:FUNBLUE_RGB_BLUE alpha:1.0];
	field.contentVerticalAlignment =  UIControlContentVerticalAlignmentCenter;
	field.delegate = self;
	
	return field;
}

- (NSString*)friendlySyncValue
{
	if ([syncValue isEqual:@"two-way"])
	{
		return NSLocalizedString(@"Two Way",@"Two Way");
	}
	else if ([syncValue isEqual:@"one-way-from-client"])
	{
		return NSLocalizedString(@"From Client",@"From Client");
	}
	else if ([syncValue isEqual:@"one-way-from-server"])
	{
		return NSLocalizedString(@"From Server",@"From Server");
	}
	else if ([syncValue isEqual:@"none"])
	{
		return NSLocalizedString(@"Disabled",@"Disabled");
	}
	else
	{
		return @"Unknown";
	}
}

- (void)updateSyncValue
{
	NSString *sync = [[FunambolAPI getInstance] getSync:@"addressbook"];
	self.syncValue = sync;
	[sync release];
}
@end