/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
#import <UIKit/UIKit.h>

#import "UIController.h"
#import "base/Log.h"
#import "spdm/DeviceManagementNode.h"
#import "customs.h"
#import "MacMappingStoreBuilder.h"
#import "spds/MappingsManager.h"

#import "main.h"

#import "FunambolAPI.h"
#import "stringUtils.h"
#import "AccountView.h"

USE_NAMESPACE

@interface FunambolDelegate ()
- (void)loadKeychainCredentials;
@end

@implementation FunambolDelegate
@synthesize window;

- (id) init
{
    if (self = [super init]) {
        credentials = nil;
    }
    
    return self;
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    // Create an instance of the UIController and show the main screen
  
    if (launchOptions) {
        NSURL *url = (NSURL*)[launchOptions valueForKey: UIApplicationLaunchOptionsURLKey];
        if (url) {
            credentials = [[url absoluteString] stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
			NSLog(@"CREDS: %@", credentials);
			AccountView *account = [[[AccountView alloc] init] autorelease];
			[account setCredentials:credentials];
			[account parseCredentials];
        }
    }
	
	[self loadKeychainCredentials];

	mainview = [[MainSingleView alloc] init];
	
	CGRect screenRect = [[UIScreen mainScreen] bounds];
    self.window = [[[UIWindow alloc] initWithFrame:screenRect] autorelease];

    navigationController = [[UINavigationController alloc] initWithRootViewController:mainview];
    [[UIController getInstance] setMainSingleView:mainview];
	
	[[UIApplication sharedApplication] setStatusBarStyle: UIStatusBarStyleBlackOpaque];
    
    // add the navigation controller's view to the window
	[self.window addSubview: navigationController.view];
	[self.window makeKeyAndVisible];

    return YES;
}

- (void)loadKeychainCredentials {
	NSString *keychainUsername = [SFHFKeychainUtils getPasswordForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil];
	NSString *keychainPassword = [SFHFKeychainUtils getPasswordForUsername:keychainUsername andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil];
	
	if (keychainUsername && keychainPassword)
	{
		[[FunambolAPI getInstance] saveUsernameConfig:keychainUsername];	
		[[FunambolAPI getInstance] savePasswordConfig:keychainPassword];
	}
}

- (void) applicationWillTerminate:(id)unused{
    
    [[FunambolAPI getInstance] stopSyncAndExit];
}

- (void)applicationWillResignActive:(id)unused{
    exit(0);
}


@end


int main(int argc, char **argv)
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    //const char* val = getHomeDir();
    LOG.setLogPath(getHomeDir());
    LOG.setLogName("/synclog.txt");
    time_t now = time(NULL);
    LOG.info("Funambol iPhone Plugin: current UTC date and time: %s",asctime(gmtime(&now)));
    DeviceManagementNode::setConfigPath(getHomeDir());
    
    MacMappingStoreBuilder* builder = new MacMappingStoreBuilder();
    MappingsManager::setBuilder(builder);
    //delete [] val;
    int retVal = UIApplicationMain(argc, argv, nil , @"FunambolDelegate");
    delete builder;
	[pool release];
	return retVal;
}
