//
//  iSubAppDelegate.m
//  iSub
//
//  Created by Ben Baron on 2/27/10.
//  Copyright Ben Baron 2010. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, 
//  are permitted provided that the following conditions are met:
//  
//  * Redistributions of source code must retain the above copyright notice, this 
//    list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice, 
//    this list of conditions and the following disclaimer in the documentation 
//    and/or other materials provided with the distribution.
//  * Neither the my name nor the names of my contributors may be used to endorse
//    or promote products derived from this software without specific prior written 
//    permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
//  DAMAGE.

#import "iSubAppDelegate.h"
#import "ArtistListViewController.h"
#import "AlbumListViewController.h"
#import "PlaylistListViewController.h"
#import "SongListViewController.h"
#import "SettingsViewController.h"
#import "StreamingPlayer.h"
#import "Subsonic.h"
#import "URLQueryStringParser.h"

@interface iSubAppDelegate ()
- (void)recoverState;
- (void)parseQueryCredentials:(NSURL*)anURL;
- (BOOL)configureUsernamePassword;
@end


@implementation iSubAppDelegate

@synthesize window;
@synthesize tabBarController;

#pragma mark -
#pragma mark Application lifecycle

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{   
	if (launchOptions)
	{
        NSURL *url = (NSURL*) [launchOptions valueForKey:UIApplicationLaunchOptionsURLKey];
        
		if (url)
		{
			[self parseQueryCredentials:url];
        }
    }
	
    [self configureUsernamePassword];

    [self recoverState];

    window.backgroundColor = [UIColor viewFlipsideBackgroundColor];
    [window addSubview:[tabBarController view]];

    [tabBarController setViewControllers:[NSArray arrayWithObjects:[ArtistListViewController navigableViewController], [AlbumListViewController navigableViewController], [SongListViewController navigableViewController], [PlaylistListViewController navigableViewController], [SettingsViewController navigableViewController], nil] animated:NO];

    [window makeKeyAndVisible];

    return YES;
}

- (BOOL)application:(UIApplication *)application handleOpenURL:(NSURL *)url
{
	if ([[url scheme] isEqual:@"x-ubuntuone-music"])
	{
		[self parseQueryCredentials:url];
		[[NSNotificationCenter defaultCenter] postNotificationName:NOTIF_reloadAccountCredentials object:nil];

		return YES;
	}
	else
	{
		return NO;
	}
}

- (void)applicationWillResignActive:(UIApplication*)application
{
	//
}

- (void)applicationDidBecomeActive:(UIApplication*)application
{
	//
}

- (void)applicationWillTerminate:(UIApplication *)application
{
	[[StreamingPlayer sharedStreamingPlayer] saveDefaults];
}

#pragma mark -
#pragma mark Helper methods

- (void)parseQueryCredentials:(NSURL*)anURL
{
	URLQueryStringParser *queryStringParser = [[URLQueryStringParser alloc] initWithURL:anURL];
	NSString *user = [queryStringParser queryStringValueForKey:@"u"];
	NSString *pass = [queryStringParser queryStringValueForKey:@"p"];
	
	if (user != nil && pass != nil)
	{
		[[Subsonic sharedSubsonic] storeUsername:user password:pass];
	}
	
	[queryStringParser release];
}

- (BOOL)configureUsernamePassword
{
	BOOL needToRetrieveCredentials = NO;
	
	NSString *username = [SFHFKeychainUtils getPasswordForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil];
	if (username)
	{
		[[Subsonic sharedSubsonic] setDefaultUserName:username];
		[[Subsonic sharedSubsonic] setDefaultPassword:[SFHFKeychainUtils getPasswordForUsername:username andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil]];
	}
	else
	{
		needToRetrieveCredentials = YES;
	}
	return needToRetrieveCredentials;
}

// Recover current state if player was interrupted
- (void)recoverState
{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	if ([[defaults objectForKey:@"recover"] isEqualToString:@"YES"])
	{
		[[StreamingPlayer sharedStreamingPlayer] recoverState:[[defaults objectForKey:@"isShuffle"] isEqualToString:@"YES"]];
		
		[defaults setObject:@"NO" forKey:@"recover"];
		[defaults synchronize];
	}
}

#pragma mark -
#pragma mark Memory management

- (void)dealloc
{
    [tabBarController release];

    [window release];

    [super dealloc];
}

@end