//
//  NSMutableArray+Extras.m
//  iSub
//
//  Created by Aaron Brethorst on 9/26/10.
//  Copyright 2010 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "NSMutableArray+Extras.h"

// helper function: get the string form of any object
static NSString *toString(id object)
{
	return [NSString stringWithFormat: @"%@", object];
}

// helper function: get the url encoded string form of any object
static NSString *urlEncode(id object)
{
	NSString *string = toString(object);
	return [string stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
}

@implementation NSMutableArray (Extras)
- (void)shuffle
{
	NSUInteger count = [self count];
    
	for (NSUInteger i = 0; i < count; ++i)
	{
        // Select a random element between i and end of array to swap with.
        int nElements = count - i;
        int n = (arc4random() % nElements) + i;
        [self exchangeObjectAtIndex:i withObjectAtIndex:n];
    }
}

- (void)addKeyValueObjectFromArray:(NSArray *)array
{
    [self addObject:[NSDictionary dictionaryWithObjects:array
                                                forKeys:[NSArray arrayWithObjects:@"key", @"value", nil]]];
}

- (NSString*)urlEncodedString
{
	NSMutableArray *parts = [NSMutableArray array];
	
	for (id object in self)
	{
        id key = [object objectForKey:@"key"];
		id value = [object objectForKey:@"value"];
		NSString *part = [NSString stringWithFormat:@"%@=%@",urlEncode(key),urlEncode(value)];
		[parts addObject:part];
	}
	return [parts componentsJoinedByString:@"&"];
}

@end
