//
//  SongUITableViewCell.m
//  iSub
//
//  Created by Ben Baron on 3/20/10.
//  Copyright 2010 Ben Baron. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, 
//  are permitted provided that the following conditions are met:
//  
//  * Redistributions of source code must retain the above copyright notice, this 
//    list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice, 
//    this list of conditions and the following disclaimer in the documentation 
//    and/or other materials provided with the distribution.
//  * Neither the my name nor the names of my contributors may be used to endorse
//    or promote products derived from this software without specific prior written 
//    permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
//  DAMAGE.

#import "SongUITableViewCell.h"

#define kEdgeColor [UIColor colorWithRed:0.8784f green:0.8784f blue:0.8784f alpha:1.0f]

@implementation SongUITableViewCell

@synthesize cacheButton, spinner, songNameLabel, positionLabel, durationLabel, nowPlaying, evenRow;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
    if ((self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]))
	{		
		self.evenRow = YES;
		
		coloredView = [[UIView alloc] initWithFrame:self.frame];
		coloredView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
		[self.contentView addSubview:coloredView];

        // XXX: This is added first so that it sits behind the cacheButton. This isn't optimal. Something better would be to delegate taps on the spinner to the cacheButton, regardless of its front-to-back ordering in the subviews.
        spinner = [[[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray] retain];
        spinner.hidesWhenStopped = YES;
        [coloredView addSubview:spinner];
		
        cacheButton = [[UIButton buttonWithType:UIButtonTypeCustom] retain];
        [cacheButton setImage:[UIImage imageNamed:@"uncached"] forState:UIControlStateNormal];
        [coloredView addSubview:self.cacheButton];
        
        songNameLabel = [[UILabel alloc] init];
		songNameLabel.font = [UIFont boldSystemFontOfSize:[UIFont systemFontSize]];
		[coloredView addSubview:self.songNameLabel];
		
		positionLabel = [[UILabel alloc] init];
		positionLabel.text = @"1";
		positionLabel.textAlignment = UITextAlignmentCenter;
		positionLabel.font = [UIFont boldSystemFontOfSize:[UIFont systemFontSize]];
		[coloredView addSubview:positionLabel];
		
		durationLabel = [[UILabel alloc] init];
		durationLabel.text = @"0:00";
		durationLabel.textAlignment = UITextAlignmentCenter;
		durationLabel.font = [UIFont boldSystemFontOfSize:[UIFont systemFontSize]];
		[coloredView addSubview:durationLabel];
		
		nowPlaying = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 13, 12)];
		nowPlaying.image = [UIImage imageNamed:@"speaker"];
		nowPlaying.backgroundColor = [UIColor clearColor];
		nowPlaying.contentMode = UIViewContentModeCenter;
		[coloredView addSubview:nowPlaying];
		
		self.textLabel.hidden = YES;
	}
	
	return self;
}

- (void)layoutSubviews 
{
    [super layoutSubviews];
	self.positionLabel.frame = CGRectMake(0, 0, 35, self.frame.size.height);
	self.songNameLabel.frame = CGRectMake(37, 0, 164, self.frame.size.height);
	self.nowPlaying.frame = CGRectMake(203, 0, 13, self.frame.size.height);
	self.durationLabel.frame = CGRectMake(218, 0, 48, self.frame.size.height);
    self.cacheButton.frame = CGRectMake(268, 0, 44, self.frame.size.height);
    self.spinner.frame = CGRectMake(282, 14, 16, 16);
    
	if (self.evenRow)
	{
		coloredView.backgroundColor = [UIColor whiteColor];
		self.backgroundColor = [UIColor whiteColor];
		self.songNameLabel.backgroundColor = [UIColor whiteColor];
		self.positionLabel.backgroundColor = [UIColor whiteColor];
		self.durationLabel.backgroundColor = [UIColor whiteColor];
	}
	else
	{
		coloredView.backgroundColor = [UIColor colorWithRed:0.9529f green:0.9529f blue:0.9529f alpha:1.0f];
		self.backgroundColor = [UIColor colorWithRed:0.9529f green:0.9529f blue:0.9529f alpha:1.0f];
		self.songNameLabel.backgroundColor = [UIColor colorWithRed:0.9529f green:0.9529f blue:0.9529f alpha:1.0f];
		self.positionLabel.backgroundColor = [UIColor colorWithRed:0.9529f green:0.9529f blue:0.9529f alpha:1.0f];
		self.durationLabel.backgroundColor = [UIColor colorWithRed:0.9529f green:0.9529f blue:0.9529f alpha:1.0f];
	}
}

- (void)dealloc
{
    [cacheButton release];
    [spinner release];
	[durationLabel release];
	[songNameLabel release];
	[positionLabel release];
    [super dealloc];
}

@end