//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "UORadialProgressControl.h"

#import <QuartzCore/QuartzCore.h>


@implementation UORadialProgressControl

@synthesize value, fillColor, strokeColor;

- (void) dealloc;
{
	self.fillColor = nil;
	self.strokeColor = nil;
	[super dealloc];
}

- (CGFloat) value;
{
	return value;
}

- (void) setValue:(CGFloat)newValue;
{
	value = newValue;
	[self setNeedsDisplay];
}

- (void) drawRect:(CGRect)rect;
{
	CGContextRef ctx = UIGraphicsGetCurrentContext();
	CGContextSetFillColorWithColor(ctx, self.fillColor.CGColor);
	CGContextSetStrokeColorWithColor(ctx, self.strokeColor.CGColor);

	CGFloat sliceAngle = self.value * (M_PI * 2.f) - (M_PI / 2.f);
	
	CGMutablePathRef path = CGPathCreateMutable();
	CGPathMoveToPoint(path, NULL, CGRectGetMidX(self.bounds), CGRectGetMidY(self.bounds));
	CGPathAddArc(path,
				 NULL,
				 CGRectGetMidX(self.bounds),
				 CGRectGetMidY(self.bounds),
				 CGRectGetWidth(self.bounds) / 2.0,
				 -M_PI / 2.f,
				 sliceAngle,
				 NO);
	CGPathCloseSubpath(path);
	CGContextAddPath(ctx, path);
	CGContextFillPath(ctx);
	CGPathRelease(path);

	CGContextSetLineWidth(ctx, 4.f); // TODO: border width should be a property
	CGContextStrokeEllipseInRect(ctx, self.bounds);
}

@end

