//
//  Album.m
//  iSub
//
//  Created by Ben Baron on 2/28/10.
//  Copyright 2010 Ben Baron. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, 
//  are permitted provided that the following conditions are met:
//  
//  * Redistributions of source code must retain the above copyright notice, this 
//    list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice, 
//    this list of conditions and the following disclaimer in the documentation 
//    and/or other materials provided with the distribution.
//  * Neither the my name nor the names of my contributors may be used to endorse
//    or promote products derived from this software without specific prior written 
//    permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
//  DAMAGE.

#import "Album.h"
#import "Artist.h"
#import "NSMutableArray+Extras.h"
#import "NSString+Extras.h"
#import "AlbumParser.h"
#import "Subsonic.h"

@interface Album ()
+ (Album*)albumMatchingPredicate:(NSPredicate*)predicate;
@end

@implementation Album
@dynamic title, artist, albumId, coverArtId;
@dynamic artistEntity, songs;

+ (BOOL)albumWithIdExists:(NSString*)anAlbumId
{
	return (nil != [Album albumWithId:anAlbumId]);
}

+ (Album*)albumWithId:(NSString*)anAlbumId
{
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"albumId == %@", anAlbumId];
	return [self albumMatchingPredicate:predicate];
}

+ (Album*)albumWithTitle:(NSString*)albumTitle artist:(NSString*)albumArtist;
{
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"title == %@ AND artist = %@", albumTitle, albumArtist];
	return [self albumMatchingPredicate:predicate];
}

+ (Album*)albumMatchingPredicate:(NSPredicate*)predicate;
{
	NSFetchRequest * fetch = [[NSFetchRequest alloc] init];
	NSEntityDescription *entityDescription = [NSEntityDescription entityForName:@"Album" inManagedObjectContext:PerThreadManagedObjectContext()];
	[fetch setEntity:entityDescription];
	[fetch setPredicate:predicate];
	NSError * error = nil;
	NSArray * matches = [PerThreadManagedObjectContext() executeFetchRequest:fetch error:&error];
	[fetch release];
	
	return [matches lastObject];
}

- (NSString *)dearticlizedTitle
{
    return [self.title dearticlizedString];
}

- (NSError*)loadSongs
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSError *error = nil;
	
	NSMutableArray *parameters = [NSMutableArray array];
	[parameters addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"id", self.albumId, nil]];
	NSURL *url = [[Subsonic sharedSubsonic] getBaseURL:@"getAlbum.view" parameters:parameters];
	NSXMLParser *xmlParser = [[NSXMLParser alloc] initWithContentsOfURL:url];
	
	AlbumParser *parser = [[AlbumParser alloc] init];
	parser.albumObjectId = [self objectID];
	
	[xmlParser setDelegate:parser];
	
	if (![xmlParser parse])
	{
		error = [[xmlParser parserError] copy];
	}
	
	[xmlParser release];
	[parser release];
	[pool release];
	
	return [error autorelease];
}

- (NSString*)description
{
	return [NSString stringWithFormat:@"%@ - %@ (%@) (art: %@) MOC %@",self.artist,self.title,self.albumId,self.coverArtId, [self managedObjectContext]];
}

- (BOOL)matchesSearchQuery:(NSString *)query
{
    NSRange titleResultsRange = [self.title rangeOfString:query options:NSCaseInsensitiveSearch];
    return titleResultsRange.length > 0;
}

- (BOOL)hasCachedSongs;
{
	NSManagedObjectContext *moc = [self managedObjectContext];
	NSEntityDescription *songEntity = [NSEntityDescription entityForName:@"Song" inManagedObjectContext:moc];
	NSFetchRequest *fetch = [[[NSFetchRequest alloc] init] autorelease];
	[fetch setEntity:songEntity];
	[fetch setPredicate:[NSPredicate predicateWithFormat:@"album = %@ and cachedSongPath != NIL", self.title]];
	
	NSArray *results = [[self managedObjectContext] executeFetchRequest:fetch error:NULL];
	return [results count] > 0;
}

@end
