//
//  Artist.m
//  iSub
//
//  Created by Ben Baron on 2/27/10.
//  Copyright 2010 Ben Baron. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, 
//  are permitted provided that the following conditions are met:
//  
//  * Redistributions of source code must retain the above copyright notice, this 
//    list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice, 
//    this list of conditions and the following disclaimer in the documentation 
//    and/or other materials provided with the distribution.
//  * Neither the my name nor the names of my contributors may be used to endorse
//    or promote products derived from this software without specific prior written 
//    permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
//  DAMAGE.

#import "Artist.h"
#import "Album.h"
#import "Subsonic.h"
#import "NSMutableArray+Extras.h"
#import "NSString+Extras.h"
#import "ArtistParser.h"

@interface Artist ()
+ (Artist*)artistMatchingPredicate:(NSPredicate*)predicate;
@end


@implementation Artist

@dynamic name;
@dynamic artistId;
@dynamic albums;
@dynamic songs;

+ (BOOL)artistWithIdExists:(NSString*)anArtistId
{
	return [Artist artistWithId:anArtistId] != nil;
}

+ (Artist*)artistWithId:(NSString*)anArtistId
{
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"artistId == %@", anArtistId];
	return [self artistMatchingPredicate:predicate];
}

+ (Artist*)artistWithName:(NSString*)artistName;
{
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name == %@", artistName];
	return [self artistMatchingPredicate:predicate];
}

+ (Artist*)artistMatchingPredicate:(NSPredicate*)predicate;
{
	NSFetchRequest * fetch = [[NSFetchRequest alloc] init];
	NSEntityDescription *entityDescription = [NSEntityDescription entityForName:@"Artist" inManagedObjectContext:PerThreadManagedObjectContext()];
	[fetch setEntity:entityDescription];
	[fetch setPredicate:predicate];
	NSError * error = nil;
	NSArray * matches = [PerThreadManagedObjectContext() executeFetchRequest:fetch error:&error];
	[fetch release];
	
	return [matches lastObject];
}

- (NSString*)description
{
	return [NSString stringWithFormat:@"%@ (%@) MOC %@",self.name, self.artistId, [self managedObjectContext]];
}

- (BOOL)matchesSearchQuery:(NSString *)query
{
    NSRange titleResultsRange = [self.name rangeOfString:query options:NSCaseInsensitiveSearch];
    return titleResultsRange.length > 0;
}

- (NSString *)dearticlizedName
{
    return [self.name dearticlizedString];
}

// Note: executes synchronously!
- (NSError*)loadAlbums
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSError *error = nil;
	
	NSMutableArray *parameters = [NSMutableArray array];
	[parameters addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"id", self.artistId, nil]];
	NSURL *url = [[Subsonic sharedSubsonic] getBaseURL:@"getMusicDirectory.view" parameters:parameters];
	NSXMLParser *xmlParser = [[NSXMLParser alloc] initWithContentsOfURL:url];
	ArtistParser *parser = [[ArtistParser alloc] init];
	parser.artistManagedObjectID = [self objectID];
	[xmlParser setDelegate:parser];
	
	if (![xmlParser parse])
	{
		error = [[xmlParser parserError] copy];
	}
	
	[xmlParser release];
	[parser release];
	[pool release];
	
	return [error autorelease];
}

- (BOOL)hasCachedSongs;
{
	NSManagedObjectContext *moc = [self managedObjectContext];
	NSEntityDescription *songEntity = [NSEntityDescription entityForName:@"Song" inManagedObjectContext:moc];
	NSFetchRequest *fetch = [[[NSFetchRequest alloc] init] autorelease];
	[fetch setEntity:songEntity];
	[fetch setPredicate:[NSPredicate predicateWithFormat:@"artist = %@ and cachedSongPath != NIL", self.name]];
	
	NSError *error = nil;
	NSArray *results = [moc executeFetchRequest:fetch error:&error];
	return [results count] > 0;
}

@end
