//
//  MOC.m
//
//  Created by Aaron Brethorst on 3/2/11.
//  Copyright 2011 Structlab LLC. All rights reserved.
//

#import "MOC.h"

//adapted from: https://gist.github.com/417195

static NSManagedObjectModel *managedObjectModel_;
static NSPersistentStoreCoordinator *persistentStoreCoordinator_;

void StoreManagedObjectContextForCurrentThread( NSManagedObjectContext * context )
{
    [[[NSThread currentThread] threadDictionary] setObject: context forKey: AQPerThreadManagedObjectContext];
}

NSManagedObjectContext * PerThreadManagedObjectContext( void )
{
    NSManagedObjectContext * result = [[[NSThread currentThread] threadDictionary] objectForKey: AQPerThreadManagedObjectContext];
    if ( result != nil )
        return ( result );
    
    NSManagedObjectContext * moc = [[NSManagedObjectContext alloc] init];
	[moc setMergePolicy: NSMergeByPropertyObjectTrumpMergePolicy];
    [moc setPersistentStoreCoordinator: GetPersistentStoreCoordinator()];
    StoreManagedObjectContextForCurrentThread( moc );
    [moc release];      // now owned by the thread dictionary
	
    return ( moc );
}

void SaveContext(void)
{
	NSError *error = nil;
	NSManagedObjectContext *managedObjectContext = PerThreadManagedObjectContext();
		
	if (managedObjectContext != nil)
	{
		if ([managedObjectContext hasChanges])
		{
			if (![managedObjectContext save:&error])
			{
				//Replace this implementation with code to handle the error appropriately.
				//abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
				NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
				abort();
			}
		} 
	}
}

NSPersistentStoreCoordinator * GetPersistentStoreCoordinator( void )
{
	if (persistentStoreCoordinator_ != nil)
	{
		return persistentStoreCoordinator_;
	}
	
	NSURL *storeURL = DatabaseFile();
	
	NSError *error = nil;
	NSInteger retries = 1;
	
	do {
		persistentStoreCoordinator_ = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:GetManagedObjectModel()];
		if (![persistentStoreCoordinator_ addPersistentStoreWithType:NSSQLiteStoreType
													   configuration:nil
																 URL:storeURL
															 options:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],NSMigratePersistentStoresAutomaticallyOption, [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption, nil]
															   error:&error])
		{
			[persistentStoreCoordinator_ release], persistentStoreCoordinator_ = nil;
			// delete store
			NSFileManager *fileManager = [[NSFileManager alloc] init];
			[fileManager removeItemAtURL:storeURL error:NULL];
			[fileManager release];
		}
	} while (retries-- > 0);
	
	if (!persistentStoreCoordinator_)
	{
		/*
		 Replace this implementation with code to handle the error appropriately.
		 
		 abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
		 
		 Typical reasons for an error here include:
		 * The persistent store is not accessible;
		 * The schema for the persistent store is incompatible with current managed object model.
		 Check the error message to determine what the actual problem was.
		 
		 
		 If the persistent store is not accessible, there is typically something wrong with the file path. Often, a file URL is pointing into the application's resources directory instead of a writeable directory.
		 
		 If you encounter schema incompatibility errors during development, you can reduce their frequency by:
		 * Simply deleting the existing store:
		 [[NSFileManager defaultManager] removeItemAtURL:storeURL error:nil]
		 
		 * Performing automatic lightweight migration by passing the following dictionary as the options parameter: 
		 [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],NSMigratePersistentStoresAutomaticallyOption, [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption, nil];
		 
		 Lightweight migration will only work for a limited set of schema changes; consult "Core Data Model Versioning and Data Migration Programming Guide" for details.
		 
		 */
		NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
		abort();
	}    
	
	return persistentStoreCoordinator_;
}

NSManagedObjectModel* GetManagedObjectModel(void)
{
    
    if (managedObjectModel_ != nil) {
        return managedObjectModel_;
    }
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"music" withExtension:@"momd"];
    managedObjectModel_ = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];    
    return managedObjectModel_;
}

void TearDownCoreData(void)
{
	[managedObjectModel_ release];
    [persistentStoreCoordinator_ release];	
}