//
//  Playlist.m
//  iSub
//
//  Created by Aaron Brethorst on 8/26/10.
//  Copyright 2010 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "Playlist.h"
#import "Subsonic.h"
#import "NSMutableArray+Extras.h"
#import "PlaylistParser.h"
#import "PlaylistSongIndex.h"
#import "Song.h"

@implementation Playlist

@dynamic playlistId, name, playlistSongIndexes;

+ (BOOL)playlistWithNameExists:(NSString *)aPlaylistName
{
	return (nil != [Playlist playlistWithName:aPlaylistName]);
}

+ (Playlist *)playlistWithName:(NSString *)aPlaylistName
{
	NSFetchRequest * fetch = [[NSFetchRequest alloc] init];
	NSEntityDescription *entityDescription = [NSEntityDescription entityForName:@"Playlist" inManagedObjectContext:PerThreadManagedObjectContext()];
	[fetch setEntity:entityDescription];
	
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name == %@", aPlaylistName];
	[fetch setPredicate:predicate];
	
	NSError * error = nil;
	NSArray * matches = [PerThreadManagedObjectContext() executeFetchRequest:fetch error:&error];
	[fetch release];
	
	if ([matches count] > 0)
	{
		return [matches objectAtIndex:0];
	}
	else
	{
		return nil;
	}
}

- (NSError*)loadSongs
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSError *error = nil;
	
	NSMutableArray *parameters = [NSMutableArray array];
	[parameters addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"id", self.playlistId, nil]];
	NSURL *url = [[Subsonic sharedSubsonic] getBaseURL:@"getPlaylist.view" parameters:parameters];
	NSXMLParser *xmlParser = [[NSXMLParser alloc] initWithContentsOfURL:url];
	
	PlaylistParser *parser = [[PlaylistParser alloc] init];
	parser.playlistObjectId = [self objectID];
	
	[xmlParser setDelegate:parser];
	
	if (![xmlParser parse])
	{
		error = [[xmlParser parserError] copy];
	}
	
	[xmlParser release];
	[parser release];
	[pool release];
	
	return [error autorelease];
}

- (NSArray *)songs
{
	// return playlistSongIndexes sorted by index
	NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"index" ascending:YES];
	NSArray *results = [[self.playlistSongIndexes sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]] valueForKey:@"songEntity"];
	return results;
}

// Do the crazy PlaylistSongIndex lookup/clobber dance inside these methods
- (void)addSong:(Song *)song
{
	PlaylistSongIndex *playlistSongIndex = [NSEntityDescription insertNewObjectForEntityForName:@"PlaylistSongIndex" inManagedObjectContext:PerThreadManagedObjectContext()];
	playlistSongIndex.playlistEntity = self;
	playlistSongIndex.songEntity = song;
	playlistSongIndex.index = [NSNumber numberWithUnsignedInt:[self.playlistSongIndexes count]];
	SaveContext();
}

- (void)removeSong:(Song *)song
{
	// Look up (potentially) all records where playlistEntity is self, and songEntity is song, and blow them away
	NSSet *condemned = [self.playlistSongIndexes filteredSetUsingPredicate:[NSPredicate predicateWithFormat:@"songEntity = %@", song]];
	for (PlaylistSongIndex *playlistSongIndex in condemned)
	{
		[PerThreadManagedObjectContext() deleteObject:playlistSongIndex];
	}
	SaveContext();
	
	[[self.playlistSongIndexes sortedArrayUsingDescriptors:[NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"index" ascending:YES]]] enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
		((PlaylistSongIndex *)obj).index = [NSNumber numberWithUnsignedInt:idx];
	}];
	SaveContext();
}

- (NSString*)description
{
	return [NSString stringWithFormat:@"%@ (%@) MOC %@", self.name, self.playlistId, [self managedObjectContext]];
}

- (BOOL)matchesSearchQuery:(NSString *)query
{
    NSRange titleResultsRange = [self.name rangeOfString:query options:NSCaseInsensitiveSearch];
    return titleResultsRange.length > 0;
}

- (BOOL)isEditable;
{
	return YES;
}

@end
