//
//  Song.m
//  iSub
//
//  Created by Ben Baron on 2/28/10.
//  Copyright 2010 Ben Baron. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, 
//  are permitted provided that the following conditions are met:
//  
//  * Redistributions of source code must retain the above copyright notice, this 
//    list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice, 
//    this list of conditions and the following disclaimer in the documentation 
//    and/or other materials provided with the distribution.
//  * Neither the my name nor the names of my contributors may be used to endorse
//    or promote products derived from this software without specific prior written 
//    permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
//  DAMAGE.

#import "Song.h"
#import "Artist.h"
#import "Album.h"
#import "NSString+Extras.h"

@implementation Song
@dynamic title, songId, artist, album, genre, coverArtId, path, duration, bitRate, track, discNumber, year, size;
@dynamic artistEntity, albumEntity, playlistSongIndexes, cachedSongPath;

+ (BOOL)songWithIdExists:(NSString*)aSongId
{
	return (nil != [Song songWithId:aSongId]);
}

+ (Song*)songWithId:(NSString*)aSongId
{
	NSFetchRequest * fetch = [[NSFetchRequest alloc] init];
	NSEntityDescription *entityDescription = [NSEntityDescription entityForName:@"Song" inManagedObjectContext:PerThreadManagedObjectContext()];
	[fetch setEntity:entityDescription];
	
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"songId == %@", aSongId];
	[fetch setPredicate:predicate];
	
	NSError * error = nil;
	NSArray * matches = [PerThreadManagedObjectContext() executeFetchRequest:fetch error:&error];
	[fetch release];
	
	if ([matches count] > 0)
	{
		return [matches objectAtIndex:0];
	}
	else
	{
		return nil;
	}
}

- (NSURL*)cachedSongURL
{
	return [CachedMusicDirectory() URLByAppendingPathComponent:self.songId];
}

- (BOOL)cachedSongExists
{
	return self.cachedSongPath != nil;
}

- (NSComparisonResult)compare:(Song*)s
{
	NSUInteger selfPathArray[] = {[self.discNumber integerValue], [self.track integerValue]};
	NSUInteger otherPathArray[] = {[s.discNumber integerValue], [s.track integerValue]};
	
	NSIndexPath *selfPath = [NSIndexPath indexPathWithIndexes:selfPathArray length:2];
	NSIndexPath *otherPath = [NSIndexPath indexPathWithIndexes:otherPathArray length:2];
	
	return [selfPath compare:otherPath];
}

- (NSString*)description
{
	NSArray *keys = [NSArray arrayWithObjects:@"title", @"songId", @"artist", @"album", @"genre", @"coverArtId", @"path", @"duration", @"bitRate", @"track", @"year", @"size", nil];
	
	NSMutableString *desc = [NSMutableString string];
	
	for (NSString *k in keys)
	{
		[desc appendFormat:@"%@:%@ :: ",k, [self valueForKey:k]];
	}
	return desc;
}

- (BOOL)matchesSearchQuery:(NSString *)query
{
    NSRange titleResultsRange = [self.title rangeOfString:query options:NSCaseInsensitiveSearch];
    return titleResultsRange.length > 0;
}

- (NSString *)dearticlizedTitle
{
    return [self.title dearticlizedString];
}

@end
