//
//  PlaylistParser.m
//  iSub
//
//  Created by Zachery Bir on 3/13/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "PlaylistParser.h"
#import "Playlist.h"
#import "Song.h"
#import "PlaylistSongIndex.h"
#import "NSDictionary+Extras.h"

@implementation PlaylistParser
@synthesize songs;
@dynamic playlistObjectId;

- (id)init 
{
	if ((self = [super init]))
	{
        songs = [[NSMutableSet alloc] init];
        numberFormatter = [[NSNumberFormatter alloc] init];
	}		
	return self;
}

- (void) subsonicErrorCode:(NSString *)errorCode message:(NSString *)message
{
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:message delegate:nil cancelButtonTitle:NSLocalizedString(@"Dismiss", @"") otherButtonTitles:nil];
	[alert show];
	[alert release];
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName attributes:(NSDictionary *)attributeDict 
{
	if ([elementName isEqualToString:@"error"])
	{
		[self subsonicErrorCode:[attributeDict objectForKey:@"code"] message:[attributeDict objectForKey:@"message"]];
	}
    else if ([elementName isEqualToString:@"entry"])
    {
		Song *song = [Song songWithId:[attributeDict objectForKey:@"id"]];
		
		if (nil == song)
		{
			song = [NSEntityDescription insertNewObjectForEntityForName:@"Song" inManagedObjectContext:PerThreadManagedObjectContext()];
			
			song.title = [attributeDict nilifiedValueForKey:@"title"];
			song.songId = [attributeDict nilifiedValueForKey:@"id"];
			song.artist = [attributeDict nilifiedValueForKey:@"artist"];
			
			song.album = [attributeDict nilifiedValueForKey:@"album"];
			song.genre = [attributeDict nilifiedValueForKey:@"genre"];
			
			song.coverArtId = [attributeDict nilifiedValueForKey:@"coverArt"];
			
			song.path = [attributeDict nilifiedValueForKey:@"path"];
			
			song.duration = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"duration"]];
			song.bitRate = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"bitRate"]];
			song.track = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"track"]];
			song.year = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"year"]];
			song.size = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"size"]];
			
		}

		PlaylistSongIndex *playlistSongIndex = [NSEntityDescription insertNewObjectForEntityForName:@"PlaylistSongIndex" inManagedObjectContext:PerThreadManagedObjectContext()];
		playlistSongIndex.playlistEntity = playlist;
		playlistSongIndex.songEntity = song;
		playlistSongIndex.index = [NSNumber numberWithUnsignedInt:[songs count]];
		[songs addObject:playlistSongIndex];
    }
	playlist.playlistSongIndexes = songs;
	SaveContext();
}

- (void) dealloc 
{
    [numberFormatter release];
    [playlist release];
    [songs release];
	[super dealloc];
}

#pragma mark -
#pragma mark Accessors

- (void)setPlaylistObjectId:(NSManagedObjectID *)oid
{
	Playlist *aPlaylist = [[PerThreadManagedObjectContext() objectWithID:oid] retain];
	[playlist release];
	playlist = aPlaylist;
}

- (NSManagedObjectID*)playlistObjectID
{
	return [playlist objectID];
}

@end
