//
//  SongListParser.m
//  iSub
//
//  Created by Zachery Bir on 4/1/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "SongListParser.h"
#import "Song.h"
#import "Album.h"
#import "Artist.h"
#import "NSDictionary+Extras.h"

@implementation SongListParser

- (id)init 
{
	if ((self = [super init]))
	{
		numberFormatter = [[NSNumberFormatter alloc] init];
	}		
	return self;
}


- (void) subsonicErrorCode:(NSString *)errorCode message:(NSString *)message
{
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:message delegate:nil cancelButtonTitle:NSLocalizedString(@"Dismiss", @"") otherButtonTitles:nil];
	[alert show];
	[alert release];
}


- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName attributes:(NSDictionary *)attributeDict 
{
	if ([elementName isEqualToString:@"error"])
	{
		[self subsonicErrorCode:[attributeDict objectForKey:@"code"] message:[attributeDict objectForKey:@"message"]];
	}
	else if ([elementName isEqualToString:@"song"]) 
	{
		if (![Song songWithIdExists:[attributeDict objectForKey:@"id"]] && ![[attributeDict objectForKey:@"title"] isEqual:@".AppleDouble"])
		{
			Song *song = [NSEntityDescription insertNewObjectForEntityForName:@"Song" inManagedObjectContext:PerThreadManagedObjectContext()];
			
			song.title = [attributeDict nilifiedValueForKey:@"title"];
			song.songId = [attributeDict nilifiedValueForKey:@"id"];
			song.artist = [attributeDict nilifiedValueForKey:@"artist"];
			song.album = [attributeDict nilifiedValueForKey:@"album"];
			song.genre = [attributeDict nilifiedValueForKey:@"genre"];
			
			song.coverArtId = [attributeDict nilifiedValueForKey:@"coverArt"];
			
			song.path = [attributeDict nilifiedValueForKey:@"path"];
			
			song.duration = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"duration"]];
			song.bitRate = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"bitRate"]];
			song.track = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"track"]];
			song.year = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"year"]];
			song.size = [numberFormatter numberFromString:[attributeDict nilifiedValueForKey:@"size"]];

			SaveContext();
		}
	}
}


- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName 
{
	//
}


- (void) dealloc 
{
	[numberFormatter release];
	[super dealloc];
}

@end