//
//  StreamingPlayer.h
//  iSub
//
//  Created by Aaron Brethorst on 9/23/10.
//  Copyright 2010 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import <Foundation/Foundation.h>
#import "AudioStreamer.h"

typedef enum {
	RepeatModeOff = 1 << 1,
	RepeatModeAll = 1 << 2,
	RepeatModeOne = 1 << 3,
} RepeatMode;

@class Song;

@interface StreamingPlayer : NSObject
{
	AudioStreamer *streamer;
	double streamerProgress;
	NSTimer *progressUpdateTimer;
	
	RepeatMode repeatMode;
	BOOL isShuffle;
	BOOL isPlaying;
	float seekTime;
	BOOL isNewSong;
	
	Song *currentSong;
	
	NSArray *currentPlaylist;
	NSMutableArray *shufflePlaylist;
	UIImage *currentCoverArt;
	
	NSURL *songUrl;
	NSURL *coverArtUrl;
}
@property(nonatomic,retain) AudioStreamer *streamer;

@property double streamerProgress;
@property RepeatMode repeatMode;
@property BOOL isShuffle;
@property BOOL isPlaying;
@property float seekTime;
@property BOOL isNewSong;
@property int currentSongIndex;

@property (nonatomic,retain) Song *currentSong;
@property (nonatomic,retain) NSArray *currentPlaylist;
@property (nonatomic,retain) NSMutableArray *shufflePlaylist;
@property (nonatomic,retain) UIImage *currentCoverArt;

@property (nonatomic,retain) NSURL *songUrl;
@property (nonatomic,retain) NSURL *coverArtUrl;

+ (StreamingPlayer*)sharedStreamingPlayer;

- (void)saveDefaults;
- (void)recoverState:(BOOL)withShuffle;

- (void)createStreamer;
- (void)destroyStreamer;
- (void)playPauseSong;
- (void)nextSong;
- (void)jumpAhead:(int)by;
- (void)nextSongAuto;
- (void)prevSong;
@end
