//
//  Subsonic.m
//  iSub
//
//  Created by Aaron Brethorst on 9/23/10.
//  Copyright 2010 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "Subsonic.h"
#import "SynthesizeSingleton.h"
#include <netinet/in.h> 
#include <netdb.h>
#include <arpa/inet.h>
#import "NSMutableArray+Extras.h"
#import "Reachability.h"

@interface Subsonic ()
@property (readwrite, retain) Reachability *reachability;
@end


@implementation Subsonic
@synthesize credsURL, defaultURL, defaultUserName, defaultPassword, cachedIP, cachedIPHour;
@synthesize reachability;
@dynamic hasCredentials;

SYNTHESIZE_SINGLETON_FOR_CLASS(Subsonic);

- (id)init
{
	if ((self = [super init]))
	{
        self.defaultURL = @"https://streaming.one.ubuntu.com/rest";
        self.credsURL = @"https://one.ubuntu.com/phones/creds/ios?scheme=x-ubuntuone-music";
		self.reachability = [Reachability reachabilityWithHostName:[[NSURL URLWithString:self.defaultURL] host]];
		[self.reachability startNotifer];
	}
	return self;
}

- (void)dealloc;
{
	self.defaultURL = nil;
	self.credsURL = nil;
	[self.reachability stopNotifer];
	self.reachability = nil;
	[super dealloc];
}

- (BOOL)hasCredentials
{
	NSString *username = [SFHFKeychainUtils getPasswordForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil];
	if (username)
	{
		NSString *password = [SFHFKeychainUtils getPasswordForUsername:username andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil];
		return (password != nil);
	}
	else
	{
		return NO;
	}
}

- (void)removeCredentials
{
	NSString *username = [[SFHFKeychainUtils getPasswordForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil] retain];
	
	if (username)
	{
		[SFHFKeychainUtils deleteItemForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil];
		[SFHFKeychainUtils deleteItemForUsername:username andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil];
	}
	
	[username release];
}

- (void)storeUsername:(NSString*)username password:(NSString*)password
{
	[SFHFKeychainUtils storeUsername:UBUNTU_ONE_DUMMY_USER_NAME andPassword:username forServiceName:UBUNTU_ONE_SERVICE_NAME updateExisting:YES error:nil];
	[SFHFKeychainUtils storeUsername:username andPassword:password forServiceName:UBUNTU_ONE_SERVICE_NAME updateExisting:YES error:nil];
	
	self.defaultUserName = username;
	self.defaultPassword = password;
}

- (NSURL *)getStreamingURLForSongId:(NSString*)songId
{
    NSMutableArray *parameters = [NSMutableArray array];
    [parameters addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"id", songId, nil]];
	return [self getBaseURL:@"stream.view" parameters:parameters];
}

- (NSURL *)getCredsURL
{
    return [NSURL URLWithString:self.credsURL];
}

- (NSURL *)getBaseURL:(NSString *)action
{
	return [self getBaseURL:action parameters:nil];
}

- (NSURL *)getBaseURL:(NSString *)action parameters:(NSArray*)extraParameters
{
	if (self.defaultUserName && self.defaultPassword)
	{
		NSMutableArray *params = [NSMutableArray arrayWithArray:extraParameters];
		
		[params addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"u", self.defaultUserName, nil]];
		[params addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"p", self.defaultPassword, nil]];
		[params addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"v", @"1.1.0", nil]];
		[params addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"c", @"iSub", nil]];
        
		NSURL *url = [NSURL URLWithString:[NSString stringWithFormat:@"%@/%@?%@", self.defaultURL, action, [params urlEncodedString]]];
        return url;
	}
	else
	{
		return nil;
	}
}

@end
