//
//  AlbumArtLoadingOperation.m
//  iSub
//
//  Created by Aaron Brethorst on 5/4/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

// adapted from http://www.dribin.org/dave/blog/archives/2009/05/05/concurrent_operations/

#import "AlbumArtLoadingOperation.h"

@implementation AlbumArtLoadingOperation
@synthesize delegate;
@synthesize data = _data;

+ (id)AlbumArtLoaderWithUrlString:(NSString *)urlString
{
    NSURL * url = [NSURL URLWithString:urlString];
    AlbumArtLoadingOperation * operation = [[self alloc] initWithUrl:url];
    return [operation autorelease];
}

- (id)initWithArtId:(NSString *)artId URL:(NSURL *)url
{
    self = [super initWithUrl:url];
    if (self == nil)
        return nil;
    
    _artId = [artId retain];
    return self;

}

- (void)dealloc
{
	[_data release];
    [_artId release];
	[super dealloc];
}

#pragma mark -
#pragma mark NSURLConnection delegate

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	[super connection:connection didReceiveResponse:response];
	
    [_data release];
    _data = [[NSMutableData alloc] init];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
	[super connection:connection didReceiveData:data];
	
    [_data appendData:data];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	UIImage *image = [[[UIImage alloc] initWithData:_data] autorelease];
	if (image)
	{
		[self.delegate imageLoaded:image forArtId:_artId];
	}
	[super connectionDidFinishLoading:connection];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	[super connection:connection didFailWithError:error];
}

@end
