//
//  DownloadOperation.m
//  iSub
//
//  Created by Aaron Brethorst on 5/12/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "DownloadOperation.h"

@implementation DownloadOperation
@synthesize filePath = _filePath;
@synthesize delegate = _delegate;

+ (id)downloadOperationWithURL:(NSURL*)anURL filePath:(NSString*)aFilePath
{
	DownloadOperation *operation = [[[DownloadOperation alloc] initWithUrl:anURL filePath:aFilePath] autorelease];
	return operation;
}

- (id)initWithUrl:(NSURL*)anUrl filePath:(NSString*)aFilePath
{
	if ((self = [super initWithUrl:anUrl]))
	{
		self.delegate = nil;
		
		self.filePath = aFilePath;
		
		[[NSFileManager defaultManager] createFileAtPath:self.filePath contents:[NSData data] attributes:nil];
		_fileHandle = [[NSFileHandle fileHandleForWritingAtPath:self.filePath] retain];
	}
	return self;
}

- (void)dealloc
{
	self.delegate = nil;
	self.filePath = nil;
	[_fileHandle release];
	[super dealloc];
}

#pragma mark -
#pragma mark NSURLConnection delegate

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	[super connection:connection didReceiveResponse:response];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
	[super connection:connection didReceiveData:data];
	
	[_fileHandle writeData:data];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	[_fileHandle closeFile];
	
	if (nil != self.delegate)
	{
		[self.delegate performSelectorOnMainThread:@selector(songDownloadedToPath:) withObject:self.filePath waitUntilDone:NO];
	}
	
	[super connectionDidFinishLoading:connection];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	[_fileHandle closeFile];
	[[NSFileManager defaultManager] removeItemAtPath:self.filePath error:nil];
	
	[super connection:connection didFailWithError:error];
}

@end

