//
//  ArtistListViewController.m
//  iSub
//
//  Created by Ben Baron on 2/27/10.
//  Copyright Ben Baron 2010. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, 
//  are permitted provided that the following conditions are met:
//  
//  * Redistributions of source code must retain the above copyright notice, this 
//    list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice, 
//    this list of conditions and the following disclaimer in the documentation 
//    and/or other materials provided with the distribution.
//  * Neither the my name nor the names of my contributors may be used to endorse
//    or promote products derived from this software without specific prior written 
//    permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
//  DAMAGE.

#import "ArtistListViewController.h"

#import "ArtistViewController.h"
#import "Artist.h"
#import "ArtistListParser.h"
#import "NSString+Extras.h"

@interface ArtistListViewController ()
- (void)configureCell:(UITableViewCell*)cell forTableView:(UITableView*)tableView indexPath:(NSIndexPath*)indexPath;
@end

@implementation ArtistListViewController

+ (UINavigationController *)navigableViewController
{
    ArtistListViewController *artistListViewController = [[[ArtistListViewController alloc] initWithTitle:NSLocalizedString(@"Artists", @"")] autorelease];
	ArtistListParser *parser = [[ArtistListParser alloc] init];
	artistListViewController.parserDelegate = parser;
	artistListViewController.viewName = @"getIndexes.view";
	
    UINavigationController *nav = [[[UINavigationController alloc] initWithRootViewController:artistListViewController] autorelease];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    nav.tabBarItem.image = [UIImage imageNamed:@"artists"];
    return nav;
}

#pragma mark - Data Loading

- (void)loadLocalData
{
	@synchronized(self.tableData)
	{
		[self.tableData removeAllObjects];
		[self.groupedTableData removeAllObjects];
		self.indexes = [NSArray array];
		
		[self.tableData addObjectsFromArray:[PerThreadManagedObjectContext() fetchObjectsForEntityName:@"Artist" withPredicate:nil]];
		
		NSMutableSet *set = [NSMutableSet set];
		
		for (Artist *a in self.tableData)
		{
			NSString *index = [[a.name dearticlizedIndex] uppercaseString];
			NSMutableArray *group = [self.groupedTableData objectForKey:index];
			
			if (nil == group)
			{
				group = [NSMutableArray array];
				[self.groupedTableData setObject:group forKey:index];
			}
			
			[group addObject:a];
			[set addObject:index];
		}
		
		self.indexes = [[set allObjects] sortedArrayUsingSelector:@selector(compare:)];
		
		indexed = ([self.tableData count] > 8);
	}
	[self.tableView reloadData];
}

- (void)updateReachability:(Reachability*)reachability;
{
	[super updateReachability:reachability];
	
	for (UITableViewCell *cell in [self.tableView visibleCells])
	{
		[self configureCell:cell forTableView:self.tableView indexPath:[self.tableView indexPathForCell:cell]];
	}
}

#pragma mark - UITableView

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath 
{
	static NSString *CellIdentifier = @"Cell";
	
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (nil == cell)
	{
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
	}
	[self configureCell:cell forTableView:tableView indexPath:indexPath];
	return cell;
}

- (void)configureCell:(UITableViewCell*)cell forTableView:(UITableView*)tableView indexPath:(NSIndexPath*)indexPath
{
	Artist *artist = nil;
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		artist = [self.searchResults objectAtIndex:indexPath.row];
		cell.accessoryType = (canStream || [artist hasCachedSongs]) ? UITableViewCellAccessoryDisclosureIndicator : UITableViewCellAccessoryNone;
	}
	else
	{
		cell.accessoryType = UITableViewCellAccessoryNone;
        if (indexed)
        {
            NSString *index = [self.indexes objectAtIndex:indexPath.section];
            artist = [[self.groupedTableData objectForKey:index] objectAtIndex:indexPath.row]; 
        }
        else
        {
            artist = [self.tableData objectAtIndex:indexPath.row];
			cell.accessoryType = (canStream || [artist hasCachedSongs]) ? UITableViewCellAccessoryDisclosureIndicator : UITableViewCellAccessoryNone;
        }
	}
	
	cell.textLabel.text = artist.name;
	if (canStream || [artist hasCachedSongs])
	{
		cell.textLabel.textColor = [UIColor blackColor];
	}
	else
	{
		cell.textLabel.textColor = [UIColor lightGrayColor];
	}
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath 
{
	Artist *selection = nil;
		
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		selection = [self.searchResults objectAtIndex:indexPath.row];
	}
	else if(indexed) 
	{	
		NSString *index = [self.indexes objectAtIndex:indexPath.section];
		selection = [[self.groupedTableData objectForKey:index] objectAtIndex:indexPath.row];
	}
    else 
    {
        selection = [self.tableData objectAtIndex:indexPath.row];
    }
	
	if (canStream || [selection hasCachedSongs])
	{
		ArtistViewController *artistViewController = [[ArtistViewController alloc] initWithStyle:UITableViewStylePlain];
		artistViewController.artist = selection;
		[self.navigationController pushViewController:artistViewController animated:YES];
		[artistViewController release];
	}
	else
	{
		[tableView deselectRowAtIndexPath:indexPath animated:YES];
	}
}

@end