//
//  PlaylistEditAlbumListViewController.m
//  iSub
//
//  Created by Zachery Bir on 4/15/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "PlaylistEditAlbumListViewController.h"
#import "PlaylistEditViewController.h"
#import "Playlist.h"
#import "Album.h"
#import "PlaylistEditAlbumViewController.h"
#import "StreamingPlayer.h"
#import "Reachability.h"


@implementation PlaylistEditAlbumListViewController

@synthesize originator;

+ (UINavigationController *)navigableViewControllerWithOriginator:(PlaylistEditViewController *)originator
{
    PlaylistEditAlbumListViewController *playlistEditAlbumListViewController = [[[PlaylistEditAlbumListViewController alloc] initWithTitle:NSLocalizedString(@"Albums", @"")] autorelease];
    playlistEditAlbumListViewController.originator = originator;
    playlistEditAlbumListViewController.navigationItem.prompt = [NSString stringWithFormat:@"%@ \"%@\".", NSLocalizedString(@"Add songs to", @""), originator.playlist.name];
    UINavigationController *nav = [[[UINavigationController alloc] initWithRootViewController:playlistEditAlbumListViewController] autorelease];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    nav.tabBarItem.image = [[UIImage imageNamed:@"albums"] autorelease];
    return nav;
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];

	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Done", @"") style:UIBarButtonItemStylePlain target:originator action:@selector(finish:)] autorelease];

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reachabilityChanged:) name:kReachabilityChangedNotification object:nil];
	[self updateReachability:[[Subsonic sharedSubsonic] reachability]];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	Album *selection = nil;
		
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		selection = [self.searchResults objectAtIndex:indexPath.row];
	}
	else if(indexed) 
	{	
		NSString *index = [self.indexes objectAtIndex:indexPath.section];
		selection = [[self.groupedTableData objectForKey:index] objectAtIndex:indexPath.row];
	}
    else 
    {
        selection = [self.tableData objectAtIndex:indexPath.row];
    }
		
	PlaylistEditAlbumViewController *albumViewController = [[PlaylistEditAlbumViewController alloc] initWithPlaylistEditViewController:self.originator];
	albumViewController.album = selection;
    albumViewController.artist = selection.artist;
	[self.navigationController pushViewController:albumViewController animated:YES];
	[albumViewController release];
}

@end
