//
//  PlaylistEditAlbumViewController.m
//  iSub
//
//  Created by Zachery Bir on 4/15/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "PlaylistEditAlbumViewController.h"
#import "PlaylistEditViewController.h"
#import "Playlist.h"
#import "Song.h"
#import "PlaylistEditSongUITableViewCell.h"
#import "NSDate+Extras.h"
#import "StreamingPlayer.h"
#import "Reachability.h"

@implementation PlaylistEditAlbumViewController
@synthesize originator;

- (id)initWithPlaylistEditViewController:(PlaylistEditViewController *)playlistEditViewController
{
    self = [super initWithStyle:UITableViewStylePlain];
    if (self)
    {
        self.originator = playlistEditViewController;
        self.navigationItem.prompt = [NSString stringWithFormat:@"%@ \"%@\".", NSLocalizedString(@"Add songs to", @""), self.originator.playlist.name];
    }
    return self;
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Done", @"") style:UIBarButtonItemStylePlain target:originator action:@selector(finish:)] autorelease];

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reachabilityChanged:) name:kReachabilityChangedNotification object:nil];
	[self updateReachability:[[Subsonic sharedSubsonic] reachability]];
}

#pragma mark - Table view data source

- (UITableViewCell *)tableView:(UITableView *)tv cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	static NSString *SongCellIdentifier = @"PlaylistEditSongCellIdentifier";
	PlaylistEditSongUITableViewCell *cell = (PlaylistEditSongUITableViewCell*)[tv dequeueReusableCellWithIdentifier:SongCellIdentifier];
	
	if (nil == cell)
	{
		cell = [[[PlaylistEditSongUITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:SongCellIdentifier] autorelease];
	}

	// explicitly re-set the default state of the checkmark
	cell.accessoryType = UITableViewCellAccessoryNone;

	Song *song = [self.songs objectAtIndex:indexPath.row];
	
	cell.songNameLabel.text = song.title;
	
	cell.positionLabel.text = [NSString stringWithFormat:@"%@", [song.track intValue] > 0 ? [song.track stringValue] : @""];

	if (song.duration && [song.duration intValue] > 0)
	{
		cell.durationLabel.text = [NSDate stringFromDuration:song.duration];
	}
	else
	{
		cell.durationLabel.text = @"";
	}

    cell.nowPlaying.hidden = YES;
	
    if ([self.originator.playlist.songs containsObject:song])
    {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    }
	
    cell.evenRow = (0 == indexPath.row % 2);
	
	return cell;
}


#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	Song *selection = [self.songs objectAtIndex:indexPath.row];
    
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];

    if (cell.accessoryType == UITableViewCellAccessoryNone) {
        [self.originator.playlist addSong:selection];
        cell.accessoryType = UITableViewCellAccessoryCheckmark;

    } else {
        [self.originator.playlist removeSong:selection];
        cell.accessoryType = UITableViewCellAccessoryNone;
    }

    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

@end
