//
//  PlaylistEditArtistListViewController.m
//  iSub
//
//  Created by Zachery Bir on 4/15/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "PlaylistEditArtistListViewController.h"
#import "PlaylistEditArtistViewController.h"
#import "PlaylistEditViewController.h"
#import "Playlist.h"
#import "Artist.h"
#import "StreamingPlayer.h"
#import "Reachability.h"


@implementation PlaylistEditArtistListViewController

@synthesize originator;

+ (UINavigationController *)navigableViewControllerWithOriginator:(PlaylistEditViewController *)originator
{
    PlaylistEditArtistListViewController *playlistEditArtistListViewController = [[[PlaylistEditArtistListViewController alloc] initWithTitle:NSLocalizedString(@"Artists", @"")] autorelease];
    playlistEditArtistListViewController.originator = originator;
    playlistEditArtistListViewController.navigationItem.prompt = [NSString stringWithFormat:@"%@ \"%@\".", NSLocalizedString(@"Add songs to", @""), originator.playlist.name];
    UINavigationController *nav = [[[UINavigationController alloc] initWithRootViewController:playlistEditArtistListViewController] autorelease];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    nav.tabBarItem.image = [[UIImage imageNamed:@"artists"] autorelease];
    return nav;
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Done", @"") style:UIBarButtonItemStylePlain target:originator action:@selector(finish:)] autorelease];

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reachabilityChanged:) name:kReachabilityChangedNotification object:nil];
	[self updateReachability:[[Subsonic sharedSubsonic] reachability]];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath 
{
	Artist *selection = nil;
		
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		selection = [self.searchResults objectAtIndex:indexPath.row];
	}
	else if(indexed) 
	{	
		NSString *index = [self.indexes objectAtIndex:indexPath.section];
		selection = [[self.groupedTableData objectForKey:index] objectAtIndex:indexPath.row];
	}
    else 
    {
        selection = [self.tableData objectAtIndex:indexPath.row];
    }
		
	PlaylistEditArtistViewController *artistViewController = [[PlaylistEditArtistViewController alloc] initWithPlaylistEditViewController:self.originator];
	artistViewController.artist = selection;
	[self.navigationController pushViewController:artistViewController animated:YES];
	[artistViewController release];
}

@end
