//
//  PlaylistEditArtistViewController.m
//  iSub
//
//  Created by Zachery Bir on 4/15/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "PlaylistEditArtistViewController.h"
#import "PlaylistEditViewController.h"
#import "PlaylistEditAlbumViewController.h"
#import "Album.h"
#import "Artist.h"
#import "Playlist.h"
#import "StreamingPlayer.h"
#import "Reachability.h"


@implementation PlaylistEditArtistViewController
@synthesize originator;

- (id)initWithPlaylistEditViewController:(PlaylistEditViewController *)playlistEditViewController
{
    self = [super initWithStyle:UITableViewStylePlain];
    if (self)
    {
        self.originator = playlistEditViewController;
        self.navigationItem.prompt = [NSString stringWithFormat:@"%@ \"%@\".", NSLocalizedString(@"Add songs to", @""), self.originator.playlist.name];
    }
    return self;
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Done", @"") style:UIBarButtonItemStylePlain target:originator action:@selector(finish:)] autorelease];

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reachabilityChanged:) name:kReachabilityChangedNotification object:nil];
	[self updateReachability:[[Subsonic sharedSubsonic] reachability]];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	if (0 == indexPath.section)
	{
		Album *selection = [self.albums objectAtIndex:indexPath.row];

		PlaylistEditAlbumViewController *albumViewController = [[PlaylistEditAlbumViewController alloc] initWithPlaylistEditViewController:self.originator];
		albumViewController.album = selection;
		albumViewController.artist = self.artist.name;
		[self.navigationController pushViewController:albumViewController animated:YES];
        [albumViewController release];
	}
	else
	{
		Song *song = [self.songs objectAtIndex:indexPath.row];
		[self didSelectSong:song withPlaylist:self.songs];
	}
}

@end
