//
//  PlaylistEditSongListViewController.m
//  iSub
//
//  Created by Zachery Bir on 4/15/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "PlaylistEditSongListViewController.h"
#import "PlaylistEditViewController.h"
#import "PlaylistSongIndex.h"
#import "Playlist.h"
#import "Song.h"
#import "StreamingPlayer.h"
#import "Reachability.h"


@implementation PlaylistEditSongListViewController
@synthesize originator;

+ (UINavigationController *)navigableViewControllerWithOriginator:(PlaylistEditViewController *)originator
{
    PlaylistEditSongListViewController *playlistEditSongListViewController = [[[PlaylistEditSongListViewController alloc] initWithTitle:NSLocalizedString(@"Songs", @"")] autorelease];
    playlistEditSongListViewController.originator = originator;
    playlistEditSongListViewController.navigationItem.prompt = [NSString stringWithFormat:@"%@ \"%@\".", NSLocalizedString(@"Add songs to", @""), originator.playlist.name];
    UINavigationController *nav = [[[UINavigationController alloc] initWithRootViewController:playlistEditSongListViewController] autorelease];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    nav.tabBarItem.image = [[UIImage imageNamed:@"songs"] autorelease];
    return nav;
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Done", @"") style:UIBarButtonItemStylePlain target:originator action:@selector(finish:)] autorelease];

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reachabilityChanged:) name:kReachabilityChangedNotification object:nil];
	[self updateReachability:[[Subsonic sharedSubsonic] reachability]];
}

#pragma mark - UITableView

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath 
{
	static NSString *CellIdentifier = @"Cell";
	
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (nil == cell)
	{
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
	}
	
	// explicitly re-set the default state of the checkmark
	cell.accessoryType = UITableViewCellAccessoryNone;
	
    Song *s = nil;
    
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		s = [self.searchResults objectAtIndex:indexPath.row];
		cell.textLabel.text = s.title;
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ - %@", s.album, s.artist];
        cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
	}
	else
	{
        if (indexed)
        {
            NSString *index = [self.indexes objectAtIndex:indexPath.section];
            s = [[self.groupedTableData objectForKey:index] objectAtIndex:indexPath.row]; 
        }
        else
        {
            s = [self.tableData objectAtIndex:indexPath.row];
        }
		cell.textLabel.text = s.title;
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ - %@", s.album, s.artist];
		cell.accessoryType = UITableViewCellAccessoryNone;
	}

    if ([self.originator.playlist.songs containsObject:s])
    {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    }
	
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath 
{
	Song *selection = nil;
		
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		selection = [self.searchResults objectAtIndex:indexPath.row];
	}
	else if(indexed) 
	{	
		NSString *index = [self.indexes objectAtIndex:indexPath.section];
		selection = [[self.groupedTableData objectForKey:index] objectAtIndex:indexPath.row];
	}
    else 
    {
        selection = [self.tableData objectAtIndex:indexPath.row];
    }
    
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];

    if (cell.accessoryType == UITableViewCellAccessoryNone) {
        [self.originator.playlist addSong:selection];
        cell.accessoryType = UITableViewCellAccessoryCheckmark;

    } else {
		
        [self.originator.playlist removeSong:selection];
        cell.accessoryType = UITableViewCellAccessoryNone;
    }

    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}


@end
