//
//  PlaylistEditViewController.m
//  iSub
//
//  Created by Zachery Bir on 4/5/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "PlaylistEditViewController.h"
#import "SubsonicTableViewController.h"
#import "ArtistListViewController.h"
#import "AlbumListViewController.h"
#import "PlaylistEditArtistListViewController.h"
#import "PlaylistEditAlbumListViewController.h"
#import "PlaylistEditSongListViewController.h"
#import "Playlist.h"
#import "Song.h"
#import "SongListParser.h"
#import "NSMutableArray+Extras.h"

@implementation PlaylistEditViewController
@synthesize playlist, originator;

+ (PlaylistEditViewController *)playlistEditViewControllerWithPlaylist:(Playlist *)playlist originator:(SubsonicTableViewController *)originator
{
    PlaylistEditViewController *playlistEditViewController = [[[PlaylistEditViewController alloc] initWithPlaylist:playlist] autorelease];
    playlistEditViewController.originator = originator;
    return playlistEditViewController;
}

- (id)initWithPlaylist:(Playlist *)aPlaylist
{
	if ((self = [super initWithNibName:nil bundle:nil]))
    {
		self.playlist = aPlaylist;
        self.originator = nil;

		// Hacky way to avoid the strange push-down of the tab bar buttons
//		self.view.frame = CGRectMake(0, 0, 320, 460);
    
		[self setViewControllers:[NSArray arrayWithObjects:[PlaylistEditArtistListViewController navigableViewControllerWithOriginator:self], [PlaylistEditAlbumListViewController navigableViewControllerWithOriginator:self], [PlaylistEditSongListViewController navigableViewControllerWithOriginator:self], nil] animated:YES];
    }
    return self;
}

#pragma mark - Memory management

- (void)dealloc
{
	RELEASE_SAFELY(playlist);
	[super dealloc];
}

#pragma mark - Button action    

- (void)finish:(id)sender
{
    NSMutableArray *extraParameters = [NSMutableArray array];
    if (nil != self.playlist.playlistId)
    {
        [extraParameters addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"playlistId", self.playlist.playlistId, nil]];
    }
    [extraParameters addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"name", self.playlist.name, nil]];
    for (Song *song in self.playlist.songs)
    {
        [extraParameters addKeyValueObjectFromArray:[NSArray arrayWithObjects:@"songId", song.songId, nil]];
    }
	NSURL *url = [[Subsonic sharedSubsonic] getBaseURL:@"createPlaylist.view" parameters:extraParameters];
    NSURLRequest *request = [[[NSURLRequest alloc] initWithURL:url] autorelease];
    NSURLConnection *connection = [NSURLConnection connectionWithRequest:request delegate:self];
    [connection start];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    [self.originator performSelector:@selector(reload:) withObject:nil afterDelay:0.5];
    self.playlist = nil;
    [self dismissModalViewControllerAnimated:YES];
}
@end