//
//  PlaylistListViewController.m
//  iSub
//
//  Created by Zachery Bir on 3/7/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "PlaylistListViewController.h"
#import "PlaylistViewController.h"
#import "PlaylistEditViewController.h"
#import "PlaylistListParser.h"
#import "Playlist.h"
#import "CachedSongsPlaylist.h"
#import "AlertPrompt.h"

@implementation PlaylistListViewController

+ (UINavigationController *)navigableViewController
{
    PlaylistListViewController *playlistListViewController = [[[PlaylistListViewController alloc] initWithTitle:NSLocalizedString(@"Playlists", @"")] autorelease];
	PlaylistListParser *parser = [[PlaylistListParser alloc] init];
	playlistListViewController.parserDelegate = parser;
	playlistListViewController.viewName = @"getPlaylists.view";
	
    UINavigationController *nav = [[[UINavigationController alloc] initWithRootViewController:playlistListViewController] autorelease];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    nav.tabBarItem.image = [UIImage imageNamed:@"playlists"];
    return nav;
}

#pragma mark - Initialization

- (id)initWithTitle:(NSString *)title
{
	if ((self = [super initWithTitle:title]))
    {
        UIView *header = [[UIView alloc] initWithFrame:CGRectMake(0,0,320,40)];
        header.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        
        UIButton *createButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
        createButton.frame = CGRectMake(5, 5, 310, 30);
        [createButton setTitle:NSLocalizedString(@"Create new playlist", @"") forState:UIControlStateNormal];
        [createButton addTarget:self action:@selector(createNewPlaylist) forControlEvents:UIControlEventTouchUpInside];
		[header addSubview:createButton];
		
        self.tableView.tableHeaderView = header;
        [header release];
    }
    return self;
}

#pragma mark - Data Loading

- (void)loadLocalData
{
	@synchronized(self.tableData)
	{
		[self.tableData removeAllObjects];
		[self.tableData addObject:[CachedSongsPlaylist cachedSongsPlaylist]];
		[self.tableData addObjectsFromArray:[PerThreadManagedObjectContext() fetchObjectsForEntityName:@"Playlist" withPredicate:nil]];

	}
	[self.tableView reloadData];
}

#pragma mark - UITableView

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		return [self.searchResults count];
	}
	else 
	{
		return [self.tableData count];
	}
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath 
{
	static NSString *CellIdentifier = @"Cell";
	
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (nil == cell)
	{
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
	}
	
	cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    Playlist *p = nil;
    
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		p = [self.searchResults objectAtIndex:indexPath.row];
	}
    else
	{
		p = [self.tableData objectAtIndex:indexPath.row]; 
	}
	
	cell.textLabel.text = p.name;
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath 
{
	Playlist *selection = nil;
		
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		selection = [self.searchResults objectAtIndex:indexPath.row];
	}
	else 
	{	
		selection = [self.tableData objectAtIndex:indexPath.row];
	}
		
	PlaylistViewController *playlistViewController = [[PlaylistViewController alloc] initWithStyle:UITableViewStylePlain playlist:selection];
    playlistViewController.playlistListViewController = self;
	[self.navigationController pushViewController:playlistViewController animated:YES];
	[playlistViewController release];
}

#pragma mark - Alert handling

- (void)alertView:(UIAlertView *)alertPrompt clickedButtonAtIndex:(NSInteger)buttonIndex
{
	if (1111111 == alertPrompt.tag && alertPrompt.cancelButtonIndex != buttonIndex)
	{
        Playlist *playlist = [NSEntityDescription insertNewObjectForEntityForName:@"Playlist" inManagedObjectContext:PerThreadManagedObjectContext()];
        playlist.name = [(AlertPrompt *)alertPrompt enteredText];
		SaveContext();
        PlaylistEditViewController *playlistEditViewController = [PlaylistEditViewController playlistEditViewControllerWithPlaylist:playlist originator:self];
        [self.navigationController presentModalViewController:playlistEditViewController animated:YES];
	}
    else
    {
        // Propagate up the alert handling in case it was a different alert
        [super alertView:alertPrompt clickedButtonAtIndex:buttonIndex];
    }
}

- (void)createNewPlaylist
{
    // Launch a dialog to capture the new playlist's name
    AlertPrompt *alert = [[[AlertPrompt alloc] initWithTitle:NSLocalizedString(@"Name New Playlist", @"")
                                                 placeholder:NSLocalizedString(@"Title", @"")
                                                     message:NSLocalizedString(@"\n", @"")
                                                    delegate:self
                                           cancelButtonTitle:NSLocalizedString(@"Cancel",@"")
                                               okButtonTitle:NSLocalizedString(@"Create",@"")] autorelease];
    alert.tag = 1111111;
    [alert show];
    return;
}

@end