//
//  SettingsViewController.m
//  iSub
//
//  Created by Ben Baron on 3/3/10.
//  Copyright 2010 Ben Baron. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, 
//  are permitted provided that the following conditions are met:
//  
//  * Redistributions of source code must retain the above copyright notice, this 
//    list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice, 
//    this list of conditions and the following disclaimer in the documentation 
//    and/or other materials provided with the distribution.
//  * Neither the my name nor the names of my contributors may be used to endorse
//    or promote products derived from this software without specific prior written 
//    permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
//  DAMAGE.

#import "SettingsViewController.h"
#import "iSubAppDelegate.h"
#import "AboutViewController.h"
#import "Subsonic.h"
#import "Downloader.h"
#import "AlbumArtLoader.h"
#import "UIAlertView+Blocks.h"
#import "RIButtonItem.h"

@interface SettingsViewController (Private)
- (void)_clearDatabaseFile:(id)sender;
- (void)_clearAlbumArtCache:(id)sender;
- (void)_clearMusicCache:(id)sender;
@end

@implementation SettingsViewController

+ (UINavigationController *)navigableViewController
{
    SettingsViewController *settingsViewController = [[[SettingsViewController alloc] initWithStyle:UITableViewStyleGrouped] autorelease];
    UINavigationController *nav = [[[UINavigationController alloc] initWithRootViewController:settingsViewController] autorelease];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    nav.tabBarItem.image = [UIImage imageNamed:@"settings"];
    return nav;
}

- (id)initWithStyle:(UITableViewStyle)style
{
	if ((self = [super initWithStyle:style]))
	{
		self.title = NSLocalizedString(@"Settings",@"");
		[[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(reloadCreds:)
                                                     name:NOTIF_reloadAccountCredentials
                                                   object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(reloadCreds:)
                                                     name:NOTIF_downloadComplete
                                                   object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(reloadCreds:)
                                                     name:NOTIF_removeCachedContent
                                                   object:nil];
	}
	return self;
}

- (void)clearAllCaches:(id)sender
{
    [self _clearAlbumArtCache:sender];
    [self _clearMusicCache:sender];
    
    [self.tableView reloadData];
}

- (void)clearAlbumArtCache:(id)sender
{
    [self _clearAlbumArtCache:sender];
    [self.tableView reloadData];
}

- (void)clearMusicCache:(id)sender
{
    [self _clearMusicCache:sender];
    [self.tableView reloadData];
}

- (void)reloadCreds:(id)sender
{
	[self.tableView reloadData];
}

- (void)viewDidLoad
{
	[super viewDidLoad];
	
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"About",@"") style:UIBarButtonItemStyleBordered target:self action:@selector(about:)] autorelease];
}

- (IBAction)about:(id)sender
{
	AboutViewController *about = [[[AboutViewController alloc] initWithNibName:@"AboutViewController" bundle:nil] autorelease];
	[self.navigationController pushViewController:about animated:YES];
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [super dealloc];
}

#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
	return 3;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (0 == section)
    {
        return 1;
    }
    else if (1 == section)
    {
        return 2;
    }
    else if (2 == section)
    {
        return 4;
    }

    return 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	if (0 == indexPath.section)
	{
		static NSString *identifier = @"ButtonCellIdentifier";
		
		UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:identifier];
		
		if (nil == cell)
		{
			cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:identifier] autorelease];
			cell.textLabel.textAlignment = UITextAlignmentCenter;
		}
		
		if ([Subsonic sharedSubsonic].hasCredentials)
		{
			cell.textLabel.text = NSLocalizedString(@"Sign Out of Ubuntu One",@"Sign Out button in SettingsViewController.m");
		}
		else
		{
			cell.textLabel.text = NSLocalizedString(@"Sign In to Ubuntu One",@"Sign In button in SettingsViewController.m");
		}

		return cell;
	}
	else if (1 == indexPath.section)
	{
		static NSString *identifier = @"CacheSizeIdentifier";
		UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:identifier];
		
		if (nil == cell)
		{
			cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:identifier] autorelease];
			cell.selectionStyle = UITableViewCellSelectionStyleNone;
		}
		
        switch (indexPath.row)
        {
            case 0:
                cell.textLabel.text = NSLocalizedString(@"Song Cache",@"");
                cell.detailTextLabel.text = [Downloader cacheSize];
                break;
            case 1:
                cell.textLabel.text = NSLocalizedString(@"Album Art Cache",@"");
                cell.detailTextLabel.text = [AlbumArtLoader cacheSize];
                break;
            default:
                break;
        }
		
		return cell;
	}
    else if (2 == indexPath.section)
    {
		static NSString *identifier = @"MetadataCountIdentifier";
		UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:identifier];
		
		if (nil == cell)
		{
			cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:identifier] autorelease];
			cell.selectionStyle = UITableViewCellSelectionStyleNone;
		}
		
        switch (indexPath.row)
        {
            case 0:
                cell.textLabel.text = NSLocalizedString(@"Artists",@"");
                cell.detailTextLabel.text = [NSString stringWithFormat:@"%d",
                                             [[PerThreadManagedObjectContext() fetchObjectsForEntityName:@"Artist"
                                                                                           withPredicate:nil] count]];
                break;
            case 1:
                cell.textLabel.text = NSLocalizedString(@"Albums",@"");
                cell.detailTextLabel.text = [NSString stringWithFormat:@"%d",
                                             [[PerThreadManagedObjectContext() fetchObjectsForEntityName:@"Album"
                                                                                           withPredicate:nil] count]];
                break;
            case 2:
                cell.textLabel.text = NSLocalizedString(@"Songs",@"");
                cell.detailTextLabel.text = [NSString stringWithFormat:@"%d",
                                             [[PerThreadManagedObjectContext() fetchObjectsForEntityName:@"Song"
                                                                                           withPredicate:nil] count]];
                break;
            case 3:
                cell.textLabel.text = NSLocalizedString(@"Playlists",@"");
                cell.detailTextLabel.text = [NSString stringWithFormat:@"%d",
                                             [[PerThreadManagedObjectContext() fetchObjectsForEntityName:@"Playlist"
                                                                                           withPredicate:nil] count]];
                break;
            default:
                break;
        }
		
		return cell;
        
    }
	else
	{ 
		return nil;
	}
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	if (0 == indexPath.section)
	{
		if ([Subsonic sharedSubsonic].hasCredentials)
		{
			[[Subsonic sharedSubsonic] removeCredentials];
            [self clearAllCaches:nil];
			[[NSNotificationCenter defaultCenter] postNotificationName:NOTIF_removeCachedContent object:nil];
		}
		else
		{
			[tableView deselectRowAtIndexPath:indexPath animated:YES];
			[[UIApplication sharedApplication] openURL:[[Subsonic sharedSubsonic] getCredsURL]];
		}		
	}
    else if (1 == indexPath.section)
    {
        RIButtonItem *cancelItem = [RIButtonItem item];
        cancelItem.label = NSLocalizedString(@"Cancel", @"");

        RIButtonItem *clearItem = [RIButtonItem item];
        clearItem.label = NSLocalizedString(@"Clear", @"");
        
        if (0 == indexPath.row)
        {
            clearItem.action = ^
            {
                [self clearMusicCache:nil];
            };

        }
        else if (1 == indexPath.row)
        {
            clearItem.action = ^
            {
                [self clearAlbumArtCache:nil];
            };
        }

        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Clear This Cache?", @"")
                                                            message:NSLocalizedString(@"This action cannot be undone.", @"")
                                                   cancelButtonItem:cancelItem otherButtonItems:clearItem, nil];
        [alertView show];
        [alertView release];

        [tableView deselectRowAtIndexPath:indexPath animated:YES];
    }
}

#pragma mark - UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    
}

@end

@implementation SettingsViewController (Private)

- (void)_clearDatabaseFile:(id)sender
{
    NSURL *databaseFile = DatabaseFile();
    NSError *error = NULL;
    [[NSFileManager defaultManager] removeItemAtURL:databaseFile error:&error];
}

- (void)_clearAlbumArtCache:(id)sender
{
    NSURL *albumArtCacheDirectory = CachedAlbumArtDirectory();
    NSError *error = NULL;
    [[NSFileManager defaultManager] removeItemAtURL:albumArtCacheDirectory error:&error];
}

- (void)_clearMusicCache:(id)sender
{
    NSURL *musicCacheDirectory = CachedMusicDirectory();
    NSError *error = NULL;
	NSFileManager *fileManager = [[NSFileManager alloc] init];
    [fileManager removeItemAtURL:musicCacheDirectory error:&error];
	[fileManager release];
	
	// reset song cache flags
	NSFetchRequest *fetch = [[[NSFetchRequest alloc] init] autorelease];
	[fetch setEntity:[NSEntityDescription entityForName:@"Song" inManagedObjectContext:PerThreadManagedObjectContext()]];
	NSArray *songs = [PerThreadManagedObjectContext() executeFetchRequest:fetch error:NULL];
	[songs setValue:nil forKey:@"cachedSongPath"];
	SaveContext();
}

@end
