//
//  SongListViewController.m
//  iSub
//
//  Created by Zachery Bir on 3/16/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "SongListViewController.h"
#import "SongViewController.h"
#import "SongListParser.h"
#import "NSString+Extras.h"

@interface SongListViewController ()
- (void)configureCell:(UITableViewCell*)cell forTableView:(UITableView*)tableView indexPath:(NSIndexPath*)indexPath;
@end

@implementation SongListViewController

+ (UINavigationController *)navigableViewController
{
    SongListViewController *songListViewController = [[[SongListViewController alloc] initWithTitle:NSLocalizedString(@"Songs", @"")] autorelease];
	SongListParser *parser = [[SongListParser alloc] init];
	songListViewController.parserDelegate = parser;
	songListViewController.viewName = @"getSongs.view";
	
    UINavigationController *nav = [[[UINavigationController alloc] initWithRootViewController:songListViewController] autorelease];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    nav.tabBarItem.image = [UIImage imageNamed:@"songs"];
    return nav;
}

#pragma mark - Data Loading

- (void)loadLocalData
{
	@synchronized(self.tableData)
	{
		[self.tableData removeAllObjects];
		[self.groupedTableData removeAllObjects];
		self.indexes = [NSArray array];
		
		[self.tableData addObjectsFromArray:[PerThreadManagedObjectContext() fetchObjectsForEntityName:@"Song" withPredicate:nil]];
		
		NSMutableSet *set = [NSMutableSet set];
		
		for (Song *s in self.tableData)
		{
			NSString *index = [[s.title dearticlizedIndex] uppercaseString];
			NSMutableArray *group = [self.groupedTableData objectForKey:index];
			
			if (nil == group)
			{
				group = [NSMutableArray array];
				[self.groupedTableData setObject:group forKey:index];
			}
			
			[group addObject:s];
			[set addObject:index];
		}
		
		self.indexes = [[set allObjects] sortedArrayUsingSelector:@selector(compare:)];
		
		if ([self.tableData count] <= 8)
		{
			indexed = NO;
		}
	}
	[self.tableView reloadData];
}

- (void)updateReachability:(Reachability *)reachability
{
	[super updateReachability:reachability];
	for (UITableViewCell *cell in [self.tableView visibleCells])
	{
		[self configureCell:cell forTableView:self.tableView indexPath:[self.tableView indexPathForCell:cell]];
	}
}

#pragma mark - UITableView

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath 
{
	static NSString *CellIdentifier = @"Cell";
	
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (nil == cell)
	{
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
	}
	[self configureCell:cell forTableView:tableView indexPath:indexPath];
	return cell;
}

- (void)configureCell:(UITableViewCell*)cell forTableView:(UITableView*)tableView indexPath:(NSIndexPath*)indexPath
{
	Song *s = nil;
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		s = [self.searchResults objectAtIndex:indexPath.row];
		cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
	}
	else
	{
		cell.accessoryType = UITableViewCellAccessoryNone;
        if (indexed)
        {
            NSString *index = [self.indexes objectAtIndex:indexPath.section];
            s = [[self.groupedTableData objectForKey:index] objectAtIndex:indexPath.row]; 
        }
        else
        {
            s = [self.tableData objectAtIndex:indexPath.row];
        }
	}

	cell.textLabel.text = s.title;
	cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ - %@", s.album, s.artist];

	if (canStream || [s cachedSongExists])
	{
		cell.textLabel.textColor = [UIColor blackColor];
	}
	else
	{
		cell.textLabel.textColor = [UIColor lightGrayColor];
		cell.accessoryType = UITableViewCellAccessoryNone;
	}
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath 
{
	Song *selection = nil;
    NSArray *playlist = self.tableData;
		
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		selection = [self.searchResults objectAtIndex:indexPath.row];
        playlist = self.searchResults;
	}
	else if(indexed) 
	{	
		NSString *index = [self.indexes objectAtIndex:indexPath.section];
		selection = [[self.groupedTableData objectForKey:index] objectAtIndex:indexPath.row];
	}
    else 
    {
        selection = [self.tableData objectAtIndex:indexPath.row];
    }
	
	if (canStream || [selection cachedSongExists])
	{
		[self didSelectSong:selection withPlaylist:playlist];
	}
	else
	{
		[tableView deselectRowAtIndexPath:indexPath animated:YES];
	}
}

@end
