//
//  SongViewController.m
//  iSub
//
//  Created by Ben Baron on 2/27/10.
//  Copyright 2010 Ben Baron. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, 
//  are permitted provided that the following conditions are met:
//  
//  * Redistributions of source code must retain the above copyright notice, this 
//    list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice, 
//    this list of conditions and the following disclaimer in the documentation 
//    and/or other materials provided with the distribution.
//  * Neither the my name nor the names of my contributors may be used to endorse
//    or promote products derived from this software without specific prior written 
//    permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
//  DAMAGE.

#import "SongViewController.h"
#import "iSubAppDelegate.h"
#import "StreamingPlayer.h"
#import "AudioStreamer.h"
#import "Album.h"
#import "Song.h"
#import <QuartzCore/CoreAnimation.h>
#import <MediaPlayer/MediaPlayer.h>
#import <CFNetwork/CFNetwork.h>
#import "NSNumber+Extras.h"

@interface SongViewController (Private)
//KVO-driven stuff
- (void)updateCurrentSong;
- (void)updatePlayPauseState;

- (void)configureInfoBox;
- (void)heartbeat:(NSTimer*)t;
- (void)createTimer;
- (void)cancelTimer;

- (void)setPlayButtonImage;
- (void)setPauseButtonImage;
- (UIBarButtonItem*)flexibleSpace;
- (UIBarButtonItem*)rewindButton;
- (UIBarButtonItem*)fastForwardButton;
- (UIBarButtonItem*)playButton;
- (UIBarButtonItem*)pauseButton;
@end

@implementation SongViewController

@synthesize musicToolbar, volumeSlider, coverArtImageView, songInfoToggleButton, backButton;
@synthesize artistLabel, albumLabel, songLabel, headerView;
@synthesize scrubberSlider, songInfoBox, elapsedLabel, remainingLabel, trackMOfNLabel, shuffleButton, repeatButton;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
	if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]))
	{
		[[StreamingPlayer sharedStreamingPlayer] addObserver:self forKeyPath:@"currentSong" options:0 context:NULL];
		[[StreamingPlayer sharedStreamingPlayer] addObserver:self forKeyPath:@"isPlaying" options:0 context:NULL];
        self.hidesBottomBarWhenPushed = YES;
	}
	return self;
}

#pragma mark -
#pragma mark KVO

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if ([keyPath isEqual:@"currentSong"])
	{
		[self updateCurrentSong];
	}
	else if ([keyPath isEqual:@"isPlaying"])
	{
		[self updatePlayPauseState];
	}
}

#pragma mark -
#pragma mark View Lifecycle

- (void)viewDidLoad
{
	[super viewDidLoad];

	self.navigationItem.titleView = self.headerView;
	
	self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc] initWithCustomView:self.backButton] autorelease];
	
	MPVolumeView *volumeView = [[[MPVolumeView alloc] initWithFrame:volumeSlider.bounds] autorelease];
	[volumeSlider addSubview:volumeView];
	[volumeView sizeToFit];
	
	loader = [[AlbumArtLoader alloc] init];
	loader.delegate = self;
	loader.imageSize = 640;

	[self configureInfoBox];
	
	[self updateCurrentSong];
}

- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	[self createTimer];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
	[self cancelTimer];
}

#pragma mark -
#pragma mark IBActions

- (IBAction)shuffle:(id)sender
{
	[StreamingPlayer sharedStreamingPlayer].isShuffle = ![StreamingPlayer sharedStreamingPlayer].isShuffle;
	
	if ([StreamingPlayer sharedStreamingPlayer].isShuffle)
	{
		[shuffleButton setImage:[UIImage imageNamed:@"shuffle-highlight"] forState:UIControlStateNormal];
	}
	else
	{
		[shuffleButton setImage:[UIImage imageNamed:@"05-shuffle"] forState:UIControlStateNormal];
	}
}

- (IBAction)repeat:(id)sender
{
	switch ([StreamingPlayer sharedStreamingPlayer].repeatMode)
	{
		case RepeatModeOff:
		{
			//transition to repeat all
			[StreamingPlayer sharedStreamingPlayer].repeatMode = RepeatModeAll;
			[repeatButton setImage:[UIImage imageNamed:@"loopback-highlight"] forState:UIControlStateNormal];
			
			break;
		}
		case RepeatModeAll:
		{
			//transition to repeat one
			[StreamingPlayer sharedStreamingPlayer].repeatMode = RepeatModeOne;
			[repeatButton setImage:[UIImage imageNamed:@"loopback-one-highlight"] forState:UIControlStateNormal];
			
			break;
		}
		case RepeatModeOne:
		{
			//transition to repeat off
			[StreamingPlayer sharedStreamingPlayer].repeatMode = RepeatModeOff;
			[repeatButton setImage:[UIImage imageNamed:@"03-loopback"] forState:UIControlStateNormal];
			break;
		}
	}
}

- (IBAction)popFromStack:(id)sender
{
	[self.navigationController popViewControllerAnimated:YES];
}

- (IBAction)songInfoToggle:(id)sender
{
	if ([self.songInfoBox superview])
	{
		[self.songInfoBox removeFromSuperview];
	}
	else
	{
		[self.view addSubview:self.songInfoBox];
	}
}

- (IBAction)playButtonPressed:(id)sender
{
	[[StreamingPlayer sharedStreamingPlayer] playPauseSong];
}

- (IBAction)prevButtonPressed:(id)sender
{
	[[StreamingPlayer sharedStreamingPlayer] prevSong];
}

- (IBAction)nextButtonPressed:(id)sender
{
	[[StreamingPlayer sharedStreamingPlayer] nextSong];
}


- (IBAction)touchedSlider
{
	[self cancelTimer];
}


- (IBAction)movingSlider
{
	CGFloat percent = scrubberSlider.value / scrubberSlider.maximumValue;
	float duration = [[StreamingPlayer sharedStreamingPlayer].currentSong.duration floatValue];
	self.elapsedLabel.text = [NSNumber minutesAndSecondsFor:(percent * duration)];
	self.remainingLabel.text = [NSNumber negativeMinutesAndSecondsFor:(duration - percent * duration)];
}


- (IBAction)movedSlider
{
	[StreamingPlayer sharedStreamingPlayer].seekTime = scrubberSlider.value;
	[[StreamingPlayer sharedStreamingPlayer].streamer seekToTime:scrubberSlider.value];
	[self createTimer];
}


#pragma mark -
#pragma mark Teardown

- (void)viewDidUnload
{
	[super viewDidUnload];

	loader.delegate = nil;
	[loader release], loader = nil;
    
	self.songInfoBox = NULL;
	self.backButton = NULL;
	self.artistLabel = NULL;
	self.albumLabel = NULL;
	self.songLabel = NULL;
	self.headerView = NULL;
	self.musicToolbar = NULL;
	self.volumeSlider = NULL;
	self.coverArtImageView = NULL;
	self.songInfoToggleButton = NULL;
	self.songInfoToggleButton = NULL;
	self.scrubberSlider = NULL;
	self.elapsedLabel = NULL;
	self.remainingLabel = NULL;
	self.trackMOfNLabel = NULL;
	self.repeatButton = NULL;
	self.shuffleButton = NULL;
	
	[heartbeat invalidate];
	heartbeat = nil;
}

- (void)dealloc
{
	[[StreamingPlayer sharedStreamingPlayer] removeObserver:self forKeyPath:@"currentSong"];
	[[StreamingPlayer sharedStreamingPlayer] removeObserver:self forKeyPath:@"isPlaying"];
	
	[super dealloc];
}

- (void)reloadImages
{
	return;
}

@end

@implementation SongViewController (Private)

- (void)createTimer
{
	heartbeat = [NSTimer scheduledTimerWithTimeInterval:0.5f target:self selector:@selector(heartbeat:) userInfo:nil repeats:YES];
}

- (void)cancelTimer
{
	[heartbeat invalidate];
	heartbeat = nil;
}

- (void)configureInfoBox
{
	scrubberSlider.backgroundColor = [UIColor clearColor];  
	UIImage *stetchLeftTrack = [[UIImage imageNamed:@"blueTrack"]
								stretchableImageWithLeftCapWidth:9.0 topCapHeight:0.0];
	UIImage *stetchRightTrack = [[UIImage imageNamed:@"whiteTrack"]
								 stretchableImageWithLeftCapWidth:9.0 topCapHeight:0.0];
	[scrubberSlider setThumbImage: [UIImage imageNamed:@"whiteSlide"] forState:UIControlStateNormal];
	[scrubberSlider setMinimumTrackImage:stetchLeftTrack forState:UIControlStateNormal];
	[scrubberSlider setMaximumTrackImage:stetchRightTrack forState:UIControlStateNormal];
	
	scrubberSlider.enabled = NO;
	
	scrubberSlider.minimumValue = 0.0f;
	
	if ([StreamingPlayer sharedStreamingPlayer].currentSong.duration)
	{
		scrubberSlider.maximumValue = [[StreamingPlayer sharedStreamingPlayer].currentSong.duration floatValue];
	}
	else
	{
		scrubberSlider.maximumValue = 100.0f;
	}
	
	self.elapsedLabel.text = @"0:00";
	self.remainingLabel.text = [[StreamingPlayer sharedStreamingPlayer].currentSong.duration negativeMinutesAndSeconds];
	
	self.trackMOfNLabel.text = [NSString stringWithFormat:@"%d of %d", [StreamingPlayer sharedStreamingPlayer].currentSongIndex+1, [[StreamingPlayer sharedStreamingPlayer].currentPlaylist count]];
	
	switch ([StreamingPlayer sharedStreamingPlayer].repeatMode)
	{
		case RepeatModeOff:
		{
			//The UI is set up for Off mode by default.
			break;
		}
		case RepeatModeOne:
		{
			[repeatButton setImage:[UIImage imageNamed:@"loopback-one-highlight"] forState:UIControlStateNormal];
			break;
		}
		case RepeatModeAll:
		{
			[repeatButton setImage:[UIImage imageNamed:@"loopback-highlight"] forState:UIControlStateNormal];
			break;
		}
	}
	
	if ([StreamingPlayer sharedStreamingPlayer].isShuffle)
	{
		[shuffleButton setImage:[UIImage imageNamed:@"shuffle-highlight"] forState:UIControlStateNormal];
	}
}

- (void)heartbeat:(NSTimer*)t
{
	self.elapsedLabel.text = [NSNumber minutesAndSecondsFor:[StreamingPlayer sharedStreamingPlayer].streamer.progress];
	int remaining = ([[StreamingPlayer sharedStreamingPlayer].currentSong.duration intValue] - [StreamingPlayer sharedStreamingPlayer].streamer.progress);
	self.remainingLabel.text = [NSNumber negativeMinutesAndSecondsFor:remaining];
	self.scrubberSlider.value = [StreamingPlayer sharedStreamingPlayer].streamer.progress;
}

- (void)updateCurrentSong
{
	if ([StreamingPlayer sharedStreamingPlayer].isNewSong || ![StreamingPlayer sharedStreamingPlayer].streamer.isPlaying)
	{
		[self setPlayButtonImage];
	}
	else
	{
		[self setPauseButtonImage];
	}
	
	if ([StreamingPlayer sharedStreamingPlayer].currentSong && [StreamingPlayer sharedStreamingPlayer].currentSong.duration)
	{
		scrubberSlider.maximumValue = [[StreamingPlayer sharedStreamingPlayer].currentSong.duration floatValue];
	}
	else
	{
		scrubberSlider.maximumValue = 100.0f;
	}
	
	self.artistLabel.text = [StreamingPlayer sharedStreamingPlayer].currentSong.artist;
	self.albumLabel.text = [StreamingPlayer sharedStreamingPlayer].currentSong.album;
	self.songLabel.text = [StreamingPlayer sharedStreamingPlayer].currentSong.title;
	self.trackMOfNLabel.text = [NSString stringWithFormat:@"%d of %d", [StreamingPlayer sharedStreamingPlayer].currentSongIndex+1, [[StreamingPlayer sharedStreamingPlayer].currentPlaylist count]];
	
	NSURL *songURL = [[Subsonic sharedSubsonic] getStreamingURLForSongId:[StreamingPlayer sharedStreamingPlayer].currentSong.songId];
	[StreamingPlayer sharedStreamingPlayer].songUrl = songURL;

    NSString *albumCoverArtId = nil;
    
    // By default, try to reuse the album's artwork, falling back on the track's coverArtId if we really need it.
    if ([StreamingPlayer sharedStreamingPlayer].currentSong.albumEntity.coverArtId)
    {
        albumCoverArtId = [StreamingPlayer sharedStreamingPlayer].currentSong.albumEntity.coverArtId;
    }
    else
    {
        albumCoverArtId = [StreamingPlayer sharedStreamingPlayer].currentSong.coverArtId;
    }

    if (albumCoverArtId)
    {
        NSError *error = nil;
        coverArtImageView.image = [loader imageForArtID:albumCoverArtId error:&error];
        
        if ([error code]) {
            [loader download:albumCoverArtId];
        }
    }
	else
	{
		coverArtImageView.image = [loader defaultAlbumArt];
	}

	[StreamingPlayer sharedStreamingPlayer].currentCoverArt = coverArtImageView.image;
}

- (void)updatePlayPauseState
{
	if ([StreamingPlayer sharedStreamingPlayer].isPlaying)
	{
		[self setPauseButtonImage];
	}
	else
	{
		[self setPlayButtonImage];
	}
}

- (void)setPlayButtonImage
{	
	NSArray *items = [NSArray arrayWithObjects:
					  [self flexibleSpace],
					  [self rewindButton],
					  [self flexibleSpace],
					  [self playButton],
					  [self flexibleSpace],
					  [self fastForwardButton],
					  [self flexibleSpace], nil];
	
	[self.musicToolbar setItems:items animated:NO];
}


- (void)setPauseButtonImage
{
	NSArray *items = [NSArray arrayWithObjects:
					  [self flexibleSpace],
					  [self rewindButton],
					  [self flexibleSpace],
					  [self pauseButton],
					  [self flexibleSpace],
					  [self fastForwardButton],
					  [self flexibleSpace], nil];
	
	[self.musicToolbar setItems:items animated:NO];
}

- (UIBarButtonItem*)flexibleSpace
{
	return [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil] autorelease];
}

- (UIBarButtonItem*)rewindButton
{
	return [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRewind target:self action:@selector(prevButtonPressed:)] autorelease];
}

- (UIBarButtonItem*)fastForwardButton
{
	return [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFastForward target:self action:@selector(nextButtonPressed:)] autorelease];
}

- (UIBarButtonItem*)playButton
{
	return [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemPlay target:self action:@selector(playButtonPressed:)] autorelease];
}

- (UIBarButtonItem*)pauseButton
{
	return [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemPause target:self action:@selector(playButtonPressed:)] autorelease];
}

@end

