//
//  SubsonicIndexedTableViewController.m
//  iSub
//
//  Created by Zachery Bir on 3/31/11.
//  Copyright 2011 Canonical. All rights reserved.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "SubsonicIndexedTableViewController.h"


@implementation SubsonicIndexedTableViewController
@synthesize indexes, groupedTableData;

- (id)initWithTitle:(NSString *)title
{
	if ((self = [super initWithTitle:title]))
    {
		self.indexes = [NSArray array];
		self.groupedTableData = [NSMutableDictionary dictionary];
        indexed = YES;
    }
    return self;
}

- (void)dealloc
{
	self.indexes = NULL;
	self.groupedTableData = NULL;
	[[NSNotificationCenter defaultCenter] removeObserver:self];
    
	[super dealloc];
}

- (void)viewDidUnload
{
	self.indexes = nil;
	self.groupedTableData = [NSMutableDictionary dictionary];

	[super viewDidUnload];
}

#pragma mark -
#pragma mark Notifications

- (void)resetCredentials:(id)sender
{
	[super resetCredentials:sender];
	self.groupedTableData = [NSMutableDictionary dictionary];
    self.indexes = [NSArray array];
	[self.tableView reloadData];
}

- (void)resetTableData:(id)sender
{
	[super resetTableData:sender];
	self.groupedTableData = [NSMutableDictionary dictionary];
    self.indexes = [NSArray array];
	[self.tableView reloadData];
}

#pragma mark - UITableView

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView 
{
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		return 1;
	}
	else
	{
        if (indexed)
        {
            return [self.indexes count];
        }
        else
        {
            return 1;
        }
	}
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		return [self.searchResults count];
	}
	else 
	{
        if (indexed)
        {
            NSString *index = [self.indexes objectAtIndex:section];
            return [[self.groupedTableData objectForKey:index] count];
        }
        else
        {
            return [self.tableData count];
        }
	}
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		return nil;
	}
	else
	{
        if (indexed)
        {
            return [self.indexes objectAtIndex:section];
        }
        else
        {
            return nil;
        }
	}
}

#pragma mark - UITableView right-side indexes

- (NSArray *)sectionIndexTitlesForTableView:(UITableView *)tableView {
	
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		return nil;
	}
	else
	{
        if (indexed)
        {
            NSMutableArray *searchIndexes = [NSMutableArray arrayWithObject:UITableViewIndexSearch];
            [searchIndexes addObjectsFromArray:self.indexes];
            return searchIndexes;
        }
        else
        {
            return nil;
        }
	}
}

- (NSInteger)tableView:(UITableView *)tableView sectionForSectionIndexTitle:(NSString *)title atIndex:(NSInteger)index {
	
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		return -1;
	}
	else if (0 == index)
	{
		[tableView scrollRectToVisible:[[tableView tableHeaderView] bounds] animated:NO];
		return -1;
	}
	else
	{
		return index - 1;
	}
}

@end
