//
//  SubsonicTableViewController.m
//  iSub
//
//  Created by Aaron Brethorst on 8/25/10.
//  Copyright 2010 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "SubsonicTableViewController.h"
#import "StreamingPlayer.h"
#import "SongViewController.h"
#import "Subsonic.h"
#import "Album.h"
#import "Reachability.h"
#import "UONetworkStatusCoordinator.h"


@interface SubsonicTableViewController ()
@property BOOL hasLoadedRemoteData;
- (void)updateReachability:(Reachability*)reachability;
@end


@implementation SubsonicTableViewController
@synthesize tableData, searchResults;
@synthesize parserDelegate, viewName;
@synthesize hasLoadedRemoteData;

#pragma mark - Initialization

- (id)initWithTitle:(NSString *)title
{
	if ((self = [super initWithNibName:@"SearchableTableViewController" bundle:nil]))
    {
		[[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(reloadCredentials:)
                                                     name:NOTIF_reloadAccountCredentials
                                                   object:nil];
        self.title = title;
        self.tableData = [NSMutableArray array];
        self.searchResults = [NSMutableArray array];
		self.parserDelegate = nil;
		self.viewName = nil;
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(resetTableData:) name:NOTIF_removeCachedContent object:nil];
		
    }
    return self;
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];

	showProgressHUD = YES;
    respondToRemoteTimer = NULL;

    if (0 == [self.tableData count])
    {
        [self loadData:NO];
    }

	if (self.searchDisplayController)
		[self.tableView setContentOffset:CGPointMake(0,self.searchDisplayController.searchBar.frame.size.height) animated:NO];
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	if ([StreamingPlayer sharedStreamingPlayer].streamer)
	{
		self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Now Playing",@"")
																				   style:UIBarButtonItemStyleBordered
																				  target:self
																				  action:@selector(nowPlayingAction:)] autorelease];
	}
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reachabilityChanged:) name:kReachabilityChangedNotification object:nil];
	[self updateReachability:[[Subsonic sharedSubsonic] reachability]];
}

- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	
    if ([[UIApplication sharedApplication] respondsToSelector:@selector(beginReceivingRemoteControlEvents)])
    {
        [[UIApplication sharedApplication] beginReceivingRemoteControlEvents];
    }
    [self becomeFirstResponder]; // this enables listening for events       
    // update the UI in case we were in the background
	
	if (![Subsonic sharedSubsonic].hasCredentials)
	{
		UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Ubuntu One",@"")
														 message:NSLocalizedString(@"Ubuntu One Music works with Ubuntu One. Find out more at http://one.ubuntu.com/. Have an account? Sign-in below.",@"")
														delegate:self
											   cancelButtonTitle:NSLocalizedString(@"Cancel",@"")
											   otherButtonTitles:NSLocalizedString(@"Sign In",@""),nil] autorelease];
		alert.tag = 1010101;
		[alert show];
	}
	
	[[NSNotificationCenter defaultCenter] postNotificationName:ASStatusChangedNotification object:self];
}

- (void)viewWillDisappear:(BOOL)animated;
{
	[super viewWillDisappear:animated];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:kReachabilityChangedNotification object:nil];
	[[UIApplication sharedApplication] endReceivingRemoteControlEvents];
	[self resignFirstResponder];
}

#pragma mark -
#pragma mark Notifications

- (void)reloadCredentials:(id)sender
{
	[self resetCredentials:sender];
	[self loadData:YES];
}

- (void)resetCredentials:(id)sender
{
	self.tableData = [NSMutableArray array];
	[self.tableView reloadData];
}

- (void)resetTableData:(id)sender
{
    self.tableData = [NSMutableArray array];
    [self.tableView reloadData];
}

- (void)reachabilityChanged:(NSNotification*)notification
{
	Reachability *reachability = [notification object];
	[self updateReachability:reachability];
}

- (void)updateReachability:(Reachability*)reachability
{
	canCache = [reachability currentReachabilityStatus] == ReachableViaWiFi;
	canStream = [reachability currentReachabilityStatus] != NotReachable;
	if (canStream && !hasLoadedRemoteData)
	{
		[self loadData:NO];
	}
}

#pragma mark -
#pragma mark Remote Control

- (BOOL)canBecomeFirstResponder
{
	return YES;
}

#pragma mark -
#pragma mark Remote Control Events - The iPod controls will send these events when the app is in the background

- (void)remoteControlReceivedWithEvent:(UIEvent *)event
{
	@synchronized(self)
	{
		if (respondToRemoteTimer)
		{
			if ([respondToRemoteTimer respondsToSelector:@selector(isValid)])
			{
				if ([respondToRemoteTimer isValid])
				{
					[respondToRemoteTimer invalidate];
					respondToRemoteTimer = NULL;
				}
			}
		}
		
		respondToRemoteTimer = [NSTimer scheduledTimerWithTimeInterval:0.25 target:self selector:@selector(respondToRemoteControl:) userInfo:event repeats:NO];
	}
}

- (void)respondToRemoteControl:(NSTimer*)timer
{
	UIEvent *event = [[timer userInfo] retain];
	
	switch (event.subtype)
	{
		case UIEventSubtypeRemoteControlPlay:
		case UIEventSubtypeRemoteControlPause:
		case UIEventSubtypeRemoteControlTogglePlayPause:
		{
			[[StreamingPlayer sharedStreamingPlayer] playPauseSong];
			break;
		}
		case UIEventSubtypeRemoteControlNextTrack:
		{
			[[StreamingPlayer sharedStreamingPlayer] nextSong];
			break;
		}
		case UIEventSubtypeRemoteControlPreviousTrack:
		{
			[[StreamingPlayer sharedStreamingPlayer] prevSong];
			break;
		}
		default:
		{
			break;
		}
	}
	
	[respondToRemoteTimer invalidate];
	respondToRemoteTimer = nil;
	[event release];
}

#pragma mark - Alert handling

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
	if (1010101 == alertView.tag && 1 == buttonIndex)
	{
		[[UIApplication sharedApplication] openURL:[[Subsonic sharedSubsonic] getCredsURL]];
	}
}

#pragma mark - Data Loading

- (void)refresh
{
	showProgressHUD = NO;
	[self loadData:YES];
}

- (IBAction)reload:(id)sender
{
	[self loadData:YES]; 
}

- (void)loadData:(BOOL)force
{
	[self showLoadingUI];
	[self loadLocalData];
	
	if ([Subsonic sharedSubsonic].hasCredentials && canStream)
	{
		hasLoadedRemoteData = YES;
		[UONetworkStatusCoordinator addNetworkActivity];
		[self performSelectorInBackground:@selector(beginLoadingRemoteData:) withObject:(force ? @"YES" : @"NO")];
	}
	else
	{
		[self finishLoadingData];
	}
}

- (void)loadLocalData
{
	//Pulls from CoreData.
}

- (void)beginLoadingRemoteData:(NSString*)force
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	NSURL *url = [[Subsonic sharedSubsonic] getBaseURL:self.viewName];
	NSXMLParser *xmlParser = [[NSXMLParser alloc] initWithContentsOfURL:url];
	
	[xmlParser setDelegate:self.parserDelegate];
	
	if (![xmlParser parse])
	{
		[self performSelectorOnMainThread:@selector(failedLoadingData:) withObject:[xmlParser parserError] waitUntilDone:NO];
	}
				
	[xmlParser release];
	
	[self performSelectorOnMainThread:@selector(loadLocalData) withObject:nil waitUntilDone:NO];
	[self performSelectorOnMainThread:@selector(finishLoadingData) withObject:nil waitUntilDone:NO];
	[pool release];
}

- (void)finishLoadingData
{
	[UONetworkStatusCoordinator removeNetworkActivity];
	[self.tableView reloadData];
	[self hideLoadingUI];
}

- (void)failedLoadingData:(NSError*)error
{
	[self hideLoadingUI];
    NSLog(@"Error loading data: %@", error);
	// TODO: Get better error messages communicated to the user.
//	[self showError:NSLocalizedString(@"Unable to load your music. Please check your Internet connection or account credentials and try again.",@"")];
}

#pragma mark - IBActions

- (IBAction)nowPlayingAction:(id)sender
{
	if ([StreamingPlayer sharedStreamingPlayer].streamer)
	{
		[StreamingPlayer sharedStreamingPlayer].isNewSong = NO;
		SongViewController *streamingPlayerViewController = [[SongViewController alloc] initWithNibName:@"SongViewController" bundle:nil];
		[self.navigationController pushViewController:streamingPlayerViewController animated:YES];
		[streamingPlayerViewController release];
	}
}

#pragma mark - UITableView

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView 
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	if (self.searchDisplayController.searchResultsTableView == tableView)
	{
		return [self.searchResults count];
	}
	else 
	{
		return [self.tableData count];
	}
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return nil;
}

- (void)didSelectSong:(Song*)song withPlaylist:(NSArray*)playlist
{
	[StreamingPlayer sharedStreamingPlayer].currentSong = song;
	[StreamingPlayer sharedStreamingPlayer].currentPlaylist = playlist;
	[StreamingPlayer sharedStreamingPlayer].isNewSong = YES;
	[StreamingPlayer sharedStreamingPlayer].isShuffle = NO;
	
	SongViewController *streamingPlayerViewController = [[[SongViewController alloc]
                                                          initWithNibName:@"SongViewController" bundle:nil] autorelease];
	
	[self.navigationController pushViewController:streamingPlayerViewController animated:YES];
	
	if ([StreamingPlayer sharedStreamingPlayer].streamer)
	{
		[[StreamingPlayer sharedStreamingPlayer].streamer stop];
	}
	
	[[StreamingPlayer sharedStreamingPlayer] playPauseSong];
}

#pragma mark - Error Messages

- (void)showMessage:(NSString*)message withTitle:(NSString*)title
{
	UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:title message:message delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
	[alert show];
}

- (void)showError:(NSString*)errorMessage
{
	[self showMessage:errorMessage withTitle:NSLocalizedString(@"Error",@"")];
}

- (void)showLoadingUI
{
	if (showProgressHUD)
	{
		[self showLoadingUIWithText:NSLocalizedString(@"Loading...",@"")];
	}
}

- (void)showLoadingUIWithText:(NSString*)text
{
	// TODO: remove the hud code altogether once we're satisfied
	// with other status indicators
//	[self.hudView show:YES];
}

- (void)hideLoadingUI
{
	if (!showProgressHUD)
	{
		[self stopLoading];
		showProgressHUD = YES;
	}
}

#pragma mark - Memory management

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

- (void)viewDidUnload
{
	RELEASE_SAFELY(searchResults);
	[super viewDidUnload];
}


- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	RELEASE_SAFELY(parserDelegate);
	RELEASE_SAFELY(viewName);
    RELEASE_SAFELY(tableData);
	RELEASE_SAFELY(searchResults);

    [super dealloc];
}

#pragma mark - UISearchDisplayController Delegate Methods

- (BOOL)searchDisplayController:(UISearchDisplayController *)controller shouldReloadTableForSearchString:(NSString *)searchString
{	
	[self.searchResults removeAllObjects];
	
	for (id d in self.tableData)
	{
		if ([d matchesSearchQuery:searchString])
			[searchResults addObject:d];
	}
	
	return YES;
}

@end

