//
//  SubsonicViewController.m
//  iSub
//
//  Created by Aaron Brethorst on 9/25/10.
//  Copyright 2010 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "SubsonicViewController.h"
#import "StreamingPlayer.h"

@implementation SubsonicViewController

@synthesize jumpBy;

- (void)viewDidLoad
{
	[super viewDidLoad];

    self.jumpBy = 0;
	remoteLock = [[NSLock alloc] init];
}

- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];

	[[UIApplication sharedApplication] beginReceivingRemoteControlEvents];
	
	[self becomeFirstResponder]; // this enables listening for events	
	
	[[NSNotificationCenter defaultCenter] postNotificationName:ASStatusChangedNotification object:self];
}

- (void)viewWillDisappear:(BOOL)animated;
{
	[super viewWillDisappear:animated];
	[[UIApplication sharedApplication] endReceivingRemoteControlEvents];
	[self resignFirstResponder];
}

- (BOOL)canBecomeFirstResponder
{
	return YES;
}

#pragma mark -
#pragma mark Remote Control Events - The iPod controls will send these events when the app is in the background

- (void)remoteControlReceivedWithEvent:(UIEvent *)event
{
	self.jumpBy = self.jumpBy + 1;
	
	if ([remoteLock tryLock])
	{
		[NSThread sleepForTimeInterval:0.25];
		
		switch (event.subtype)
		{
			case UIEventSubtypeRemoteControlPlay:
			case UIEventSubtypeRemoteControlPause:
			case UIEventSubtypeRemoteControlTogglePlayPause:
			{
				[[StreamingPlayer sharedStreamingPlayer] playPauseSong];
				break;
			}
			case UIEventSubtypeRemoteControlNextTrack:
			{
				[[StreamingPlayer sharedStreamingPlayer] jumpAhead:self.jumpBy];
				break;
			}
			case UIEventSubtypeRemoteControlPreviousTrack:
			{
				[[StreamingPlayer sharedStreamingPlayer] prevSong];
				break;
			}
			default:
			{
				break;
			}
		}
		
		self.jumpBy = 0;
		
		[remoteLock unlock];
	}
}

- (void)dealloc
{
	RELEASE_SAFELY(remoteLock);	
	[super dealloc];
}

@end
